@@ -18,6 +18,8 @@
 #include <stdlib.h>
 #include <assert.h>
 
+#include <libavutil/common.h>
+
 #include "common/common.h"
 #include "common/msg.h"
 #include "chmap.h"
@@ -68,6 +70,7 @@ static const char *const std_layout_names[][2] = {
     {"quad",            "fl-fr-bl-br"},
     {"quad(side)",      "fl-fr-sl-sr"},
     {"3.1",             "fl-fr-fc-lfe"},
+    {"3.1(back)",       "fl-fr-lfe-bc"}, // not in lavc
     {"5.0",             "fl-fr-fc-bl-br"},
     {"5.0(alsa)",       "fl-fr-bl-br-fc"}, // not in lavc
     {"5.0(side)",       "fl-fr-fc-sl-sr"},
@@ -81,9 +84,11 @@ static const char *const std_layout_names[][2] = {
     {"hexagonal",       "fl-fr-fc-bl-br-bc"},
     {"6.1",             "fl-fr-fc-lfe-bc-sl-sr"},
     {"6.1(back)",       "fl-fr-fc-lfe-bl-br-bc"}, // lavc calls this "6.1" too
+    {"6.1(top)",        "fl-fr-fc-lfe-bl-br-tc"}, // not in lavc
     {"6.1(front)",      "fl-fr-lfe-flc-frc-sl-sr"},
     {"7.0",             "fl-fr-fc-bl-br-sl-sr"},
     {"7.0(front)",      "fl-fr-fc-flc-frc-sl-sr"},
+    {"7.0(rear)",       "fl-fr-fc-bl-br-sdl-sdr"}, // not in lavc
     {"7.1",             "fl-fr-fc-lfe-bl-br-sl-sr"},
     {"7.1(alsa)",       "fl-fr-bl-br-fc-lfe-sl-sr"}, // not in lavc
     {"7.1(wide)",       "fl-fr-fc-lfe-bl-br-flc-frc"},
@@ -95,7 +100,7 @@ static const char *const std_layout_names[][2] = {
     {0}
 };
 
-static const struct mp_chmap default_layouts[MP_NUM_CHANNELS + 1] = {
+static const struct mp_chmap default_layouts[] = {
     {0},                                        // empty
     MP_CHMAP_INIT_MONO,                         // mono
     MP_CHMAP2(FL, FR),                          // stereo
@@ -215,11 +220,11 @@ void mp_chmap_fill_na(struct mp_chmap *map, int num)
 // mp_chmap_is_valid(dst) will return false.
 void mp_chmap_from_channels(struct mp_chmap *dst, int num_channels)
 {
-    if (num_channels < 0 || num_channels > MP_NUM_CHANNELS) {
-        *dst = (struct mp_chmap) {0};
-    } else {
+    *dst = (struct mp_chmap) {0};
+    if (num_channels >= 0 && num_channels < MP_ARRAY_SIZE(default_layouts))
         *dst = default_layouts[num_channels];
-    }
+    if (!dst->num)
+        mp_chmap_set_unknown(dst, num_channels);
 }
 
 // Try to do what mplayer/mplayer2/mpv did before channel layouts were
@@ -382,8 +387,8 @@ void mp_chmap_get_reorder(int src[MP_NUM_CHANNELS], const struct mp_chmap *from,
         return;
     }
 
-    for (int n = 0; n < from->num; n++) {
-        for (int i = 0; i < to->num; i++) {
+    for (int n = 0; n < to->num; n++) {
+        for (int i = 0; i < from->num; i++) {
             if (to->speaker[n] == from->speaker[i]) {
                 src[n] = i;
                 break;
@@ -395,20 +400,12 @@ void mp_chmap_get_reorder(int src[MP_NUM_CHANNELS], const struct mp_chmap *from,
         assert(src[n] < 0 || (to->speaker[n] == from->speaker[src[n]]));
 }
 
-static int popcount64(uint64_t bits)
-{
-    int r = 0;
-    for (int n = 0; n < 64; n++)
-        r += !!(bits & (1ULL << n));
-    return r;
-}
-
 // Return the number of channels only in a.
 int mp_chmap_diffn(const struct mp_chmap *a, const struct mp_chmap *b)
 {
     uint64_t a_mask = mp_chmap_to_lavc_unchecked(a);
     uint64_t b_mask = mp_chmap_to_lavc_unchecked(b);
-    return popcount64((a_mask ^ b_mask) & a_mask);
+    return av_popcount64((a_mask ^ b_mask) & a_mask);
 }
 
 // Returns something like "fl-fr-fc". If there's a standard layout in lavc
@@ -511,6 +508,25 @@ bool mp_chmap_from_str(struct mp_chmap *dst, bstr src)
     return true;
 }
 
+// Output a human readable "canonical" channel map string. Converting this from
+// a string back to a channel map can yield a different map, but the string
+// looks nicer. E.g. "fc-fl-fr-na" becomes "3.0".
+char *mp_chmap_to_str_hr_buf(char *buf, size_t buf_size, const struct mp_chmap *src)
+{
+    struct mp_chmap map = *src;
+    mp_chmap_remove_na(&map);
+    for (int n = 0; std_layout_names[n][0]; n++) {
+        struct mp_chmap s;
+        if (mp_chmap_from_str(&s, bstr0(std_layout_names[n][0])) &&
+            mp_chmap_equals_reordered(&s, &map))
+        {
+            map = s;
+            break;
+        }
+    }
+    return mp_chmap_to_str_buf(buf, buf_size, &map);
+}
+
 void mp_chmap_print_help(struct mp_log *log)
 {
     mp_info(log, "Speakers:\n");
