@@ -31,70 +31,37 @@
 #include "af.h"
 
 // Static list of filters
-extern const struct af_info af_info_dummy;
 extern const struct af_info af_info_delay;
 extern const struct af_info af_info_channels;
 extern const struct af_info af_info_format;
-extern const struct af_info af_info_force;
 extern const struct af_info af_info_volume;
 extern const struct af_info af_info_equalizer;
 extern const struct af_info af_info_pan;
-extern const struct af_info af_info_surround;
-extern const struct af_info af_info_sub;
-extern const struct af_info af_info_export;
 extern const struct af_info af_info_drc;
-extern const struct af_info af_info_extrastereo;
 extern const struct af_info af_info_lavcac3enc;
 extern const struct af_info af_info_lavrresample;
-extern const struct af_info af_info_sweep;
-extern const struct af_info af_info_hrtf;
-extern const struct af_info af_info_ladspa;
-extern const struct af_info af_info_center;
-extern const struct af_info af_info_sinesuppress;
-extern const struct af_info af_info_karaoke;
 extern const struct af_info af_info_scaletempo;
 extern const struct af_info af_info_bs2b;
 extern const struct af_info af_info_lavfi;
-extern const struct af_info af_info_convert24;
-extern const struct af_info af_info_convertsignendian;
 extern const struct af_info af_info_rubberband;
 
 static const struct af_info *const filter_list[] = {
-    &af_info_dummy,
     &af_info_delay,
     &af_info_channels,
     &af_info_format,
     &af_info_volume,
     &af_info_equalizer,
     &af_info_pan,
-    &af_info_surround,
-    &af_info_sub,
-    &af_info_export,
     &af_info_drc,
-    &af_info_extrastereo,
     &af_info_lavcac3enc,
     &af_info_lavrresample,
-    &af_info_sweep,
-    &af_info_hrtf,
-#if HAVE_LADSPA
-    &af_info_ladspa,
-#endif
 #if HAVE_RUBBERBAND
     &af_info_rubberband,
 #endif
-    &af_info_center,
-    &af_info_sinesuppress,
-    &af_info_karaoke,
     &af_info_scaletempo,
-#if HAVE_LIBBS2B
-    &af_info_bs2b,
-#endif
 #if HAVE_LIBAVFILTER
     &af_info_lavfi,
 #endif
-    // Must come last, because they're fallback format conversion filter
-    &af_info_convert24,
-    &af_info_convertsignendian,
     NULL
 };
 
@@ -192,24 +159,11 @@ static struct af_instance *af_create(struct af_stream *s, char *name,
         MP_ERR(s, "Couldn't find audio filter '%s'.\n", name);
         return NULL;
     }
-    const struct af_info *info = desc.p;
-    /* Make sure that the filter is not already in the list if it is
-       non-reentrant */
-    if (info->flags & AF_FLAGS_NOT_REENTRANT) {
-        for (struct af_instance *cur = s->first; cur; cur = cur->next) {
-            if (cur->info == info) {
-                MP_ERR(s, "There can only be one "
-                       "instance of the filter '%s' in each stream\n", name);
-                return NULL;
-            }
-        }
-    }
-
     MP_VERBOSE(s, "Adding filter %s \n", name);
 
     struct af_instance *af = talloc_zero(NULL, struct af_instance);
     *af = (struct af_instance) {
-        .info = info,
+        .info = desc.p,
         .data = talloc_zero(af, struct mp_audio),
         .log = mp_log_new(af, s->log, name),
         .replaygain_data = s->replaygain_data,
@@ -334,89 +288,9 @@ static void af_print_filter_chain(struct af_stream *s, struct af_instance *at,
     MP_MSG(s, msg_level, "  [ao] %s\n", mp_audio_config_to_str(&s->output));
 }
 
-static int af_count_filters(struct af_stream *s)
-{
-    int count = 0;
-    for (struct af_instance *af = s->first; af; af = af->next)
-        count++;
-    return count;
-}
-
-// Finds the first conversion filter on the way from srcfmt to dstfmt.
-// Conversions form a DAG: each node is a format/filter pair, and possible
-// conversions are edges. We search the DAG for the shortest path.
-// Some cases visit the same filter multiple times, but with different formats
-// (like u24le->s8), so one node per format or filter separate is not enough.
-// Returns the filter and dest. format for the first conversion step.
-// (So we know what conversion filter with what format to insert next.)
-static char *af_find_conversion_filter(int srcfmt, int *dstfmt)
-{
-#define MAX_NODES (64 * 32)
-    int num_fmt = 0, num_filt = 0;
-    for (int n = 0; filter_list[n]; n++)
-        num_filt = n + 1;
-    for (int n = 0; af_fmtstr_table[n].format; n++)
-        num_fmt = n + 1;
-
-    int num_nodes = num_fmt * num_filt;
-    assert(num_nodes < MAX_NODES);
-
-    bool visited[MAX_NODES] = {0};
-    unsigned char distance[MAX_NODES];
-    short previous[MAX_NODES] = {0};
-    for (int n = 0; n < num_nodes; n++) {
-        distance[n] = 255;
-        if (af_fmtstr_table[n % num_fmt].format == srcfmt)
-            distance[n] = 0;
-    }
-
-    while (1) {
-        int next = -1;
-        for (int n = 0; n < num_nodes; n++) {
-            if (!visited[n] && (next < 0 || (distance[n] < distance[next])))
-                next = n;
-        }
-        if (next < 0 || distance[next] == 255)
-            return NULL;
-        visited[next] = true;
-
-        int fmt = next % num_fmt;
-        if (af_fmtstr_table[fmt].format == *dstfmt) {
-            // Best match found
-            for (int cur = next; cur >= 0; cur = previous[cur] - 1) {
-                if (distance[cur] == 1) {
-                    *dstfmt = af_fmtstr_table[cur % num_fmt].format;
-                    return (char *)filter_list[cur / num_fmt]->name;
-                }
-            }
-            return NULL;
-        }
-
-        for (int n = 0; filter_list[n]; n++) {
-            const struct af_info *af = filter_list[n];
-            if (!af->test_conversion)
-                continue;
-            for (int i = 0; af_fmtstr_table[i].format; i++) {
-                if (i != fmt && af->test_conversion(af_fmtstr_table[fmt].format,
-                                                    af_fmtstr_table[i].format))
-                {
-                    int other = n * num_fmt + i;
-                    int ndist = distance[next] + 1;
-                    if (ndist < distance[other]) {
-                        distance[other] = ndist;
-                        previous[other] = next + 1;
-                    }
-                }
-            }
-        }
-    }
-    assert(0);
-#undef MAX_NODES
-}
-
 static bool af_is_conversion_filter(struct af_instance *af)
 {
-    return af && af->info->test_conversion != NULL;
+    return af && strcmp(af->info->name, "lavrresample") == 0;
 }
 
 // in is what af can take as input - insert a conversion filter if the actual
@@ -436,21 +310,24 @@ static int af_fix_format_conversion(struct af_stream *s,
     if (actual.format == in.format)
         return AF_FALSE;
     int dstfmt = in.format;
-    char *filter = af_find_conversion_filter(actual.format, &dstfmt);
-    if (!filter)
+    char *filter = "lavrresample";
+    if (!af_lavrresample_test_conversion(actual.format, dstfmt))
         return AF_ERROR;
     if (strcmp(filter, prev->info->name) == 0) {
         if (prev->control(prev, AF_CONTROL_SET_FORMAT, &dstfmt) == AF_OK) {
             *p_af = prev;
             return AF_OK;
         }
+        return AF_ERROR;
     }
     struct af_instance *new = af_prepend(s, af, filter, NULL);
     if (new == NULL)
         return AF_ERROR;
     new->auto_inserted = true;
-    if (AF_OK != (rv = new->control(new, AF_CONTROL_SET_FORMAT, &dstfmt)))
+    if (AF_OK != (rv = new->control(new, AF_CONTROL_SET_FORMAT, &dstfmt))) {
+        af_remove(s, new);
         return rv;
+    }
     *p_af = new;
     return AF_OK;
 }
@@ -528,8 +405,8 @@ static int af_reinit(struct af_stream *s)
     // Start with the second filter, as the first filter is the special input
     // filter which needs no initialization.
     struct af_instance *af = s->first->next;
-    // Up to 7 retries per filter (channel, rate, 5x format conversions)
-    int max_retry = af_count_filters(s) * 7;
+    // Up to 4 retries per filter (channel, rate, format conversions)
+    int max_retry = 4;
     int retry = 0;
     while (af) {
         if (retry >= max_retry)
@@ -575,8 +452,8 @@ static int af_reinit(struct af_stream *s)
             int fmt_in1 = af->prev->data->format;
             int fmt_in2 = in.format;
             if (af_fmt_is_valid(fmt_in1) && af_fmt_is_valid(fmt_in2)) {
-                bool spd1 = AF_FORMAT_IS_IEC61937(fmt_in1);
-                bool spd2 = AF_FORMAT_IS_IEC61937(fmt_in2);
+                bool spd1 = af_fmt_is_spdif(fmt_in1);
+                bool spd2 = af_fmt_is_spdif(fmt_in2);
                 if (spd1 != spd2 && af->next) {
                     MP_WARN(af, "Filter %s apparently cannot be used due to "
                                 "spdif passthrough - removing it.\n",
@@ -601,6 +478,8 @@ static int af_reinit(struct af_stream *s)
                    af->info->name, rv);
             goto error;
         }
+        if (af && !af->auto_inserted)
+            retry = 0;
     }
 
     /* Set previously unset fields in s->output to those of the filter chain
@@ -733,7 +612,7 @@ struct af_instance *af_add(struct af_stream *s, char *name, char *label,
         return NULL;
     new->label = talloc_strdup(new, label);
 
-    // Reinitalize the filter list
+    // Reinitialize the filter list
     if (af_reinit(s) != AF_OK) {
         af_remove_by_label(s, label);
         return NULL;
@@ -803,6 +682,18 @@ void af_control_all(struct af_stream *s, int cmd, void *arg)
         af->control(af, cmd, arg);
 }
 
+int af_control_by_label(struct af_stream *s, int cmd, void *arg, bstr label)
+{
+    char *label_str = bstrdup0(NULL, label);
+    struct af_instance *cur = af_find_by_label(s, label_str);
+    talloc_free(label_str);
+    if (cur) {
+        return cur->control ? cur->control(cur, cmd, arg) : CONTROL_NA;
+    } else {
+        return CONTROL_UNKNOWN;
+    }
+}
+
 // Used by filters to add a filtered frame to the output queue.
 // Ownership of frame is transferred from caller to the filter chain.
 void af_add_output_frame(struct af_instance *af, struct mp_audio *frame)
@@ -832,6 +723,16 @@ static struct mp_audio *af_dequeue_output_frame(struct af_instance *af)
     return res;
 }
 
+static void read_remaining(struct af_instance *af)
+{
+    int num_frames;
+    do {
+        num_frames = af->num_out_queued;
+        if (!af->filter_out || af->filter_out(af) < 0)
+            break;
+    } while (num_frames != af->num_out_queued);
+}
+
 static int af_do_filter(struct af_instance *af, struct mp_audio *frame)
 {
     if (frame)
@@ -871,6 +772,7 @@ int af_output_frame(struct af_stream *s, bool eof)
             // Flush remaining frames on EOF, but do that only if the previous
             // filters have been flushed (i.e. they have no more output).
             if (eof && !last) {
+                read_remaining(cur);
                 int r = af_do_filter(cur, NULL);
                 if (r < 0)
                     return r;
