@@ -101,7 +101,7 @@ static int control(struct ao *ao, enum aocontrol cmd, void *arg)
         long get_vol, set_vol;
         float f_multi;
 
-        if (AF_FORMAT_IS_SPECIAL(ao->format))
+        if (!af_fmt_is_pcm(ao->format))
             return CONTROL_FALSE;
 
         snd_mixer_selem_id_alloca(&sid);
@@ -204,22 +204,26 @@ alsa_error:
 }
 
 static const int mp_to_alsa_format[][2] = {
-    {AF_FORMAT_S8,          SND_PCM_FORMAT_S8},
     {AF_FORMAT_U8,          SND_PCM_FORMAT_U8},
-    {AF_FORMAT_U16,         SND_PCM_FORMAT_U16},
     {AF_FORMAT_S16,         SND_PCM_FORMAT_S16},
-    {AF_FORMAT_U32,         SND_PCM_FORMAT_U32},
     {AF_FORMAT_S32,         SND_PCM_FORMAT_S32},
-    {AF_FORMAT_U24,
-            MP_SELECT_LE_BE(SND_PCM_FORMAT_U24_3LE, SND_PCM_FORMAT_U24_3BE)},
     {AF_FORMAT_S24,
             MP_SELECT_LE_BE(SND_PCM_FORMAT_S24_3LE, SND_PCM_FORMAT_S24_3BE)},
     {AF_FORMAT_FLOAT,       SND_PCM_FORMAT_FLOAT},
+    {AF_FORMAT_DOUBLE,      SND_PCM_FORMAT_FLOAT64},
     {AF_FORMAT_UNKNOWN,     SND_PCM_FORMAT_UNKNOWN},
 };
 
 static int find_alsa_format(int af_format)
 {
+    if (af_fmt_is_spdif(af_format)) {
+        if (af_format == AF_FORMAT_S_MP3) {
+            return SND_PCM_FORMAT_MPEG;
+        } else {
+            return SND_PCM_FORMAT_S16;
+        }
+    }
+
     af_format = af_fmt_from_planar(af_format);
     for (int n = 0; mp_to_alsa_format[n][0] != AF_FORMAT_UNKNOWN; n++) {
         if (mp_to_alsa_format[n][0] == af_format)
@@ -251,6 +255,8 @@ static const int alsa_to_mp_channels[][2] = {
     {SND_CHMAP_TRL,     MP_SP(TBL)},
     {SND_CHMAP_TRR,     MP_SP(TBR)},
     {SND_CHMAP_TRC,     MP_SP(TBC)},
+    {SND_CHMAP_RRC,     MP_SP(SDR)},
+    {SND_CHMAP_RLC,     MP_SP(SDL)},
     {SND_CHMAP_MONO,    MP_SP(FC)},
     {SND_CHMAP_NA,      MP_SPEAKER_ID_NA},
     {SND_CHMAP_LAST,    MP_SPEAKER_ID_COUNT}
@@ -304,6 +310,8 @@ static bool query_chmaps(struct ao *ao, struct mp_chmap *chmap)
         mp_chmap_from_alsa(&entry, &maps[i]->map);
 
         if (mp_chmap_is_valid(&entry)) {
+            if (maps[i]->type == SND_CHMAP_TYPE_VAR)
+                mp_chmap_reorder_norm(&entry);
             MP_VERBOSE(ao, "Got supported channel map: %s (type %s)\n",
                        mp_chmap_to_str(&entry),
                        snd_pcm_chmap_type_name(maps[i]->type));
@@ -375,8 +383,9 @@ static char *append_params(void *ta_parent, const char *device, const char *p)
 static int try_open_device(struct ao *ao, const char *device)
 {
     struct priv *p = ao->priv;
+    int err;
 
-    if (AF_FORMAT_IS_IEC61937(ao->format)) {
+    if (af_fmt_is_spdif(ao->format)) {
         void *tmp = talloc_new(NULL);
         char *params = talloc_asprintf(tmp,
                         "AES0=%d,AES1=%d,AES2=0,AES3=%d",
@@ -385,15 +394,29 @@ static int try_open_device(struct ao *ao, const char *device)
                         map_iec958_srate(ao->samplerate));
         const char *ac3_device = append_params(tmp, device, params);
         MP_VERBOSE(ao, "opening device '%s' => '%s'\n", device, ac3_device);
-        int err = snd_pcm_open
-                    (&p->alsa, ac3_device, SND_PCM_STREAM_PLAYBACK, 0);
+        err = snd_pcm_open(&p->alsa, ac3_device, SND_PCM_STREAM_PLAYBACK, 0);
+        if (err < 0) {
+            // Some spdif-capable devices do not accept the AES0 parameter,
+            // and instead require the iec958 pseudo-device (they will play
+            // noise otherwise). Unfortunately, ALSA gives us no way to map
+            // these devices, so try it for the default device only.
+            bstr dev;
+            bstr_split_tok(bstr0(device), ":", &dev, &(bstr){0});
+            if (bstr_equals0(dev, "default")) {
+                ac3_device = append_params(tmp, "iec958", params);
+                MP_VERBOSE(ao, "got error %d; opening iec fallback device '%s'\n",
+                           err, ac3_device);
+                err = snd_pcm_open
+                            (&p->alsa, ac3_device, SND_PCM_STREAM_PLAYBACK, 0);
+            }
+        }
         talloc_free(tmp);
-        if (!err)
-            return 0;
+    } else {
+        MP_VERBOSE(ao, "opening device '%s'\n", device);
+        err = snd_pcm_open(&p->alsa, device, SND_PCM_STREAM_PLAYBACK, 0);
     }
 
-    MP_VERBOSE(ao, "opening device '%s'\n", device);
-    return snd_pcm_open(&p->alsa, device, SND_PCM_STREAM_PLAYBACK, 0);
+    return err;
 }
 
 static void uninit(struct ao *ao)
@@ -419,8 +442,6 @@ static int init_device(struct ao *ao, bool second_try)
     int err;
 
     const char *device = "default";
-    if (AF_FORMAT_IS_IEC61937(ao->format))
-        device = "iec958";
     if (ao->device)
         device = ao->device;
     if (p->cfg_device && p->cfg_device[0])
@@ -441,38 +462,28 @@ static int init_device(struct ao *ao, bool second_try)
     err = snd_pcm_hw_params_any(p->alsa, alsa_hwparams);
     CHECK_ALSA_ERROR("Unable to get initial parameters");
 
-    if (AF_FORMAT_IS_IEC61937(ao->format)) {
-        if (ao->format == AF_FORMAT_S_MP3) {
-            p->alsa_fmt = SND_PCM_FORMAT_MPEG;
-        } else {
-            p->alsa_fmt = SND_PCM_FORMAT_S16;
-        }
-    } else {
+    int try_formats[AF_FORMAT_COUNT];
+    af_get_best_sample_formats(ao->format, try_formats);
+    for (int n = 0; try_formats[n]; n++) {
+        ao->format = try_formats[n];
         p->alsa_fmt = find_alsa_format(ao->format);
-    }
-    if (p->alsa_fmt == SND_PCM_FORMAT_UNKNOWN) {
-        p->alsa_fmt = SND_PCM_FORMAT_S16;
-        ao->format = AF_FORMAT_S16;
+        if (snd_pcm_hw_params_test_format(p->alsa, alsa_hwparams, p->alsa_fmt) >= 0)
+            break;
     }
 
-    err = snd_pcm_hw_params_test_format(p->alsa, alsa_hwparams, p->alsa_fmt);
-    if (err < 0) {
-        if (AF_FORMAT_IS_IEC61937(ao->format))
-            CHECK_ALSA_ERROR("Unable to set IEC61937 format");
-        MP_INFO(ao, "Format %s is not supported by hardware, trying default.\n",
-                af_fmt_to_str(ao->format));
-        p->alsa_fmt = SND_PCM_FORMAT_S16;
-        ao->format = AF_FORMAT_S16;
+    if (!ao->format) {
+        MP_ERR(ao, "Can't find appropriate sample format.\n");
+        goto alsa_error;
     }
 
     err = snd_pcm_hw_params_set_format(p->alsa, alsa_hwparams, p->alsa_fmt);
     CHECK_ALSA_ERROR("Unable to set format");
 
-    snd_pcm_access_t access = AF_FORMAT_IS_PLANAR(ao->format)
+    snd_pcm_access_t access = af_fmt_is_planar(ao->format)
                                     ? SND_PCM_ACCESS_RW_NONINTERLEAVED
                                     : SND_PCM_ACCESS_RW_INTERLEAVED;
     err = snd_pcm_hw_params_set_access(p->alsa, alsa_hwparams, access);
-    if (err < 0 && AF_FORMAT_IS_PLANAR(ao->format)) {
+    if (err < 0 && af_fmt_is_planar(ao->format)) {
         ao->format = af_fmt_from_planar(ao->format);
         access = SND_PCM_ACCESS_RW_INTERLEAVED;
         err = snd_pcm_hw_params_set_access(p->alsa, alsa_hwparams, access);
@@ -480,8 +491,11 @@ static int init_device(struct ao *ao, bool second_try)
     CHECK_ALSA_ERROR("Unable to set access type");
 
     struct mp_chmap dev_chmap = ao->channels;
-    if (AF_FORMAT_IS_IEC61937(ao->format) || p->cfg_ignore_chmap) {
+    if (af_fmt_is_spdif(ao->format) || p->cfg_ignore_chmap) {
         dev_chmap.num = 0; // disable chmap API
+    } else if (dev_chmap.num == 1 && dev_chmap.speaker[0] == MP_SPEAKER_ID_FC) {
+        // As yet another ALSA API inconsistency, mono is not reported correctly.
+        dev_chmap.num = 0;
     } else if (query_chmaps(ao, &dev_chmap)) {
         ao->channels = dev_chmap;
     } else {
@@ -505,8 +519,8 @@ static int init_device(struct ao *ao, bool second_try)
         mp_chmap_from_channels_alsa(&ao->channels, num_channels);
         if (!mp_chmap_is_valid(&ao->channels))
             mp_chmap_from_channels(&ao->channels, 2);
-        MP_ERR(ao, "Couldn't get requested number of channels (%d), fallback "
-               "to %s.\n", req, mp_chmap_to_str(&ao->channels));
+        MP_ERR(ao, "Asked for %d channels, got %d - fallback to %s.\n", req,
+               num_channels, mp_chmap_to_str(&ao->channels));
     }
 
     // Some ALSA drivers have broken delay reporting, so disable the ALSA
@@ -556,10 +570,11 @@ static int init_device(struct ao *ao, bool second_try)
 
         err = snd_pcm_set_chmap(p->alsa, alsa_chmap);
         if (err == -ENXIO) {
-            // I consider this an ALSA bug: the channel map was reported as
-            // supported, but we still can't set it. It happens virtually
-            // always with dmix, though.
-            MP_VERBOSE(ao, "Device does not support requested channel map (%s)\n",
+            // A device my not be able to set any channel map, even channel maps
+            // that were reported as supported. This is either because the ALSA
+            // device is broken (dmix), or because the driver has only 1
+            // channel map per channel count, and setting the map is not needed.
+            MP_VERBOSE(ao, "device returned ENXIO when setting channel map %s\n",
                        mp_chmap_to_str(&dev_chmap));
         } else {
             CHECK_ALSA_WARN("Channel map setup failed");
@@ -581,7 +596,7 @@ static int init_device(struct ao *ao, bool second_try)
 
         if (p->cfg_ignore_chmap) {
             MP_VERBOSE(ao, "user set ignore-chmap; ignoring the channel map.\n");
-        } else if (AF_FORMAT_IS_IEC61937(ao->format)) {
+        } else if (af_fmt_is_spdif(ao->format)) {
             MP_VERBOSE(ao, "using spdif passthrough; ignoring the channel map.\n");
         } else if (mp_chmap_is_valid(&chmap)) {
             // Is it one that contains NA channels?
@@ -590,17 +605,18 @@ static int init_device(struct ao *ao, bool second_try)
 
             if (mp_chmap_is_valid(&without_na) &&
                 !mp_chmap_equals(&without_na, &chmap) &&
+                !mp_chmap_equals(&chmap, &ao->channels) &&
                 !second_try)
             {
                 // Sometimes, ALSA will advertise certain chmaps, but it's not
                 // possible to set them. This can happen with dmix: as of
                 // alsa 1.0.28, dmix can do stereo only, but advertises the
                 // surround chmaps of the underlying device. In this case,
-                // requesting e.g. 5.1 will fail, but it will still allow
-                // setting 6 channels. Then it will return something like
+                // e.g. setting 6 channels will succeed, but requesting  5.1
+                // afterwards will fail. Then it will return something like
                 // "FL FR NA NA NA NA" as channel map. This means we would
                 // have to pad stereo output to 6 channels with silence, which
-                // is way too complicated in the general case. You can't change
+                // would require lots of extra processing. You can't change
                 // the number of channels to 2 either, because the hw params
                 // are already set! So just fuck it and reopen the device with
                 // the chmap "cleaned out" of NA entries.
@@ -625,8 +641,10 @@ static int init_device(struct ao *ao, bool second_try)
         }
 
         // mpv and ALSA use different conventions for mono
-        if (ao->channels.num == 1)
+        if (ao->channels.num == 1) {
+            MP_VERBOSE(ao, "assuming we actually got MONO from ALSA.\n");
             ao->channels.speaker[0] = MP_SP(FC);
+        }
 
         free(alsa_chmap);
     }
@@ -830,7 +848,7 @@ static int play(struct ao *ao, void **data, int samples, int flags)
         return 0;
 
     do {
-        if (AF_FORMAT_IS_PLANAR(ao->format)) {
+        if (af_fmt_is_planar(ao->format)) {
             res = snd_pcm_writen(p->alsa, data, samples);
         } else {
             res = snd_pcm_writei(p->alsa, data[0], samples);
@@ -904,15 +922,18 @@ static void list_devs(struct ao *ao, struct ao_device_list *list)
         char *name = snd_device_name_get_hint(hints[n], "NAME");
         char *desc = snd_device_name_get_hint(hints[n], "DESC");
         char *io = snd_device_name_get_hint(hints[n], "IOID");
-        if (io && strcmp(io, "Output") != 0)
-            continue;
-        char desc2[1024];
-        snprintf(desc2, sizeof(desc2), "%s", desc ? desc : "");
-        for (int i = 0; desc2[i]; i++) {
-            if (desc2[i] == '\n')
-                desc2[i] = '/';
+        if (io && strcmp(io, "Output") == 0) {
+            char desc2[1024];
+            snprintf(desc2, sizeof(desc2), "%s", desc ? desc : "");
+            for (int i = 0; desc2[i]; i++) {
+                if (desc2[i] == '\n')
+                    desc2[i] = '/';
+            }
+            ao_device_list_add(list, ao, &(struct ao_device_desc){name, desc2});
         }
-        ao_device_list_add(list, ao, &(struct ao_device_desc){name, desc2});
+        free(name);
+        free(desc);
+        free(io);
     }
 
     snd_device_name_free_hint(hints);
