@@ -25,18 +25,22 @@
 #include "options/m_option.h"
 #include "misc/ring.h"
 #include "common/msg.h"
-#include "audio/out/ao_coreaudio_properties.h"
-#include "audio/out/ao_coreaudio_utils.h"
+#include "ao_coreaudio_chmap.h"
+#include "ao_coreaudio_properties.h"
+#include "ao_coreaudio_utils.h"
 
 struct priv {
     AudioDeviceID device;
     AudioUnit audio_unit;
 
     uint64_t hw_latency_us;
-};
 
-bool ca_layout_to_mp_chmap(struct ao *ao, AudioChannelLayout *layout,
-                           struct mp_chmap *chmap);
+    AudioStreamBasicDescription original_asbd;
+    AudioStreamID original_asbd_stream;
+
+    int change_physical_format;
+    int exclusive;
+};
 
 static int64_t ca_get_hardware_latency(struct ao *ao) {
     struct priv *p = ao->priv;
@@ -52,12 +56,8 @@ static int64_t ca_get_hardware_latency(struct ao *ao) {
             &size);
     CHECK_CA_ERROR("cannot get audio unit latency");
 
-    uint32_t frames = 0;
-    err = CA_GET_O(p->device, kAudioDevicePropertyLatency, &frames);
-    CHECK_CA_ERROR("cannot get device latency");
-
     uint64_t audiounit_latency_us = audiounit_latency_sec * 1e6;
-    uint64_t device_latency_us    = ca_frames_to_us(ao, frames);
+    uint64_t device_latency_us    = ca_get_device_latency_us(ao, p->device);
 
     MP_VERBOSE(ao, "audiounit latency [us]: %lld\n", audiounit_latency_us);
     MP_VERBOSE(ao, "device latency [us]: %lld\n", device_latency_us);
@@ -74,11 +74,14 @@ static OSStatus render_cb_lpcm(void *ctx, AudioUnitRenderActionFlags *aflags,
 {
     struct ao *ao   = ctx;
     struct priv *p  = ao->priv;
-    AudioBuffer buf = buffer_list->mBuffers[0];
+    void *planes[MP_NUM_CHANNELS] = {0};
+
+    for (int n = 0; n < ao->num_planes; n++)
+        planes[n] = buffer_list->mBuffers[n].mData;
 
     int64_t end = mp_time_us();
     end += p->hw_latency_us + ca_get_latency(ts) + ca_frames_to_us(ao, frames);
-    ao_read_data(ao, &buf.mData, frames, end);
+    ao_read_data(ao, planes, frames, end);
     return noErr;
 }
 
@@ -121,8 +124,8 @@ static int control(struct ao *ao, enum aocontrol cmd, void *arg)
     return CONTROL_UNKNOWN;
 }
 
-static bool init_chmap(struct ao *ao);
 static bool init_audiounit(struct ao *ao, AudioStreamBasicDescription asbd);
+static void init_physical_format(struct ao *ao);
 
 static bool reinit_device(struct ao *ao) {
     struct priv *p = ao->priv;
@@ -143,8 +146,10 @@ coreaudio_error:
 
 static int init(struct ao *ao)
 {
-    if (AF_FORMAT_IS_IEC61937(ao->format)) {
-        MP_WARN(ao, "detected IEC61937, redirecting to coreaudio_exclusive\n");
+    struct priv *p = ao->priv;
+
+    if (!af_fmt_is_pcm(ao->format) || p->exclusive) {
+        MP_VERBOSE(ao, "redirecting to coreaudio_exclusive\n");
         ao->redirect = "coreaudio_exclusive";
         return CONTROL_ERROR;
     }
@@ -152,10 +157,11 @@ static int init(struct ao *ao)
     if (!reinit_device(ao))
         goto coreaudio_error;
 
-    if (!init_chmap(ao))
-        goto coreaudio_error;
+    if (p->change_physical_format)
+        init_physical_format(ao);
 
-    ao->format = af_fmt_from_planar(ao->format);
+    if (!ca_init_chmap(ao, p->device))
+        goto coreaudio_error;
 
     AudioStreamBasicDescription asbd;
     ca_fill_asbd(ao, &asbd);
@@ -169,95 +175,71 @@ coreaudio_error:
     return CONTROL_ERROR;
 }
 
-static AudioChannelLayout* ca_query_layout(struct ao *ao, void *talloc_ctx)
+static void init_physical_format(struct ao *ao)
 {
     struct priv *p = ao->priv;
-    OSStatus err;
-    uint32_t psize;
-    AudioChannelLayout *r = NULL;
+    OSErr err;
 
-    AudioObjectPropertyAddress p_addr = (AudioObjectPropertyAddress) {
-        .mSelector = kAudioDevicePropertyPreferredChannelLayout,
-        .mScope    = kAudioDevicePropertyScopeOutput,
-        .mElement  = kAudioObjectPropertyElementWildcard,
-    };
+    AudioStreamBasicDescription asbd;
+    ca_fill_asbd(ao, &asbd);
 
-    err = AudioObjectGetPropertyDataSize(p->device, &p_addr, 0, NULL, &psize);
-    CHECK_CA_ERROR("could not get device preferred layout (size)");
+    AudioStreamID *streams;
+    size_t n_streams;
 
-    r = talloc_size(talloc_ctx, psize);
+    err = CA_GET_ARY_O(p->device, kAudioDevicePropertyStreams,
+                       &streams, &n_streams);
+    CHECK_CA_ERROR("could not get number of streams");
 
-    err = AudioObjectGetPropertyData(p->device, &p_addr, 0, NULL, &psize, r);
-    CHECK_CA_ERROR("could not get device preferred layout (get)");
+    MP_VERBOSE(ao, "Found %zd substream(s).\n", n_streams);
 
-coreaudio_error:
-    return r;
-}
+    for (int i = 0; i < n_streams; i++) {
+        AudioStreamRangedDescription *formats;
+        size_t n_formats;
 
-static AudioChannelLayout* ca_query_stereo_layout(struct ao *ao, void *talloc_ctx)
-{
-    struct priv *p = ao->priv;
-    OSStatus err;
-    const int nch = 2;
-    uint32_t channels[nch];
-    AudioChannelLayout *r = NULL;
+        MP_VERBOSE(ao, "Looking at formats in substream %d...\n", i);
 
-    AudioObjectPropertyAddress p_addr = (AudioObjectPropertyAddress) {
-        .mSelector = kAudioDevicePropertyPreferredChannelsForStereo,
-        .mScope    = kAudioDevicePropertyScopeOutput,
-        .mElement  = kAudioObjectPropertyElementWildcard,
-    };
+        err = CA_GET_ARY(streams[i], kAudioStreamPropertyAvailablePhysicalFormats,
+                         &formats, &n_formats);
 
-    uint32_t psize = sizeof(channels);
-    err = AudioObjectGetPropertyData(p->device, &p_addr, 0, NULL, &psize, channels);
-    CHECK_CA_ERROR("could not get device preferred stereo layout");
+        if (!CHECK_CA_WARN("could not get number of stream formats"))
+            continue; // try next one
 
-    psize = sizeof(AudioChannelLayout) + nch * sizeof(AudioChannelDescription);
-    r = talloc_zero_size(talloc_ctx, psize);
-    r->mChannelLayoutTag = kAudioChannelLayoutTag_UseChannelDescriptions;
-    r->mNumberChannelDescriptions = nch;
 
-    AudioChannelDescription desc = {0};
-    desc.mChannelFlags = kAudioChannelFlags_AllOff;
+        uint32_t direction;
+        err = CA_GET(streams[i], kAudioStreamPropertyDirection, &direction);
+        CHECK_CA_ERROR("could not get stream direction");
+        if (direction != 0) {
+            MP_VERBOSE(ao, "Not an output stream.\n");
+            continue;
+        }
 
-    for(int i = 0; i < nch; i++) {
-        desc.mChannelLabel = channels[i];
-        r->mChannelDescriptions[i] = desc;
+        AudioStreamBasicDescription best_asbd = {0};
+
+        for (int j = 0; j < n_formats; j++) {
+            AudioStreamBasicDescription *stream_asbd = &formats[j].mFormat;
+
+            ca_print_asbd(ao, "- ", stream_asbd);
+
+            if (!best_asbd.mFormatID || ca_asbd_is_better(&asbd, &best_asbd,
+                                                          stream_asbd))
+                best_asbd = *stream_asbd;
+        }
+
+        if (best_asbd.mFormatID) {
+            p->original_asbd_stream = streams[i];
+            err = CA_GET(p->original_asbd_stream,
+                         kAudioStreamPropertyPhysicalFormat,
+                         &p->original_asbd);
+            CHECK_CA_WARN("could not get current physical stream format");
+
+            if (!ca_change_physical_format_sync(ao, streams[i], best_asbd))
+                p->original_asbd = (AudioStreamBasicDescription){0};
+            break;
+        }
     }
 
 coreaudio_error:
-    return r;
-}
-
-static bool init_chmap(struct ao *ao)
-{
-    struct priv *p = ao->priv;
-    void *ta_ctx = talloc_new(NULL);
-
-    struct mp_chmap_sel chmap_sel = {.tmp = p};
-    struct mp_chmap chmap = {0};
-
-    AudioChannelLayout *ml = ca_query_layout(ao, ta_ctx);
-    if (ml && ca_layout_to_mp_chmap(ao, ml, &chmap))
-        mp_chmap_sel_add_map(&chmap_sel, &chmap);
-
-    AudioChannelLayout *sl = ca_query_stereo_layout(ao, ta_ctx);
-    if (sl && ca_layout_to_mp_chmap(ao, sl, &chmap))
-        mp_chmap_sel_add_map(&chmap_sel, &chmap);
-
-    talloc_free(ta_ctx);
-
-    if (!ao_chmap_sel_adjust(ao, &chmap_sel, &ao->channels)) {
-        MP_ERR(ao, "could not select a suitable channel map among the "
-                   "hardware supported ones. Make sure to configure your "
-                   "output device correctly in 'Audio MIDI Setup.app'\n");
-        goto coreaudio_error;
-    }
-
-    return true;
-
-coreaudio_error:
-    return false;
+    return;
 }
 
 static bool init_audiounit(struct ao *ao, AudioStreamBasicDescription asbd)
@@ -350,13 +332,23 @@ static void uninit(struct ao *ao)
     AudioOutputUnitStop(p->audio_unit);
     AudioUnitUninitialize(p->audio_unit);
     AudioComponentInstanceDispose(p->audio_unit);
+
+    if (p->original_asbd.mFormatID) {
+        OSStatus err = CA_SET(p->original_asbd_stream,
+                              kAudioStreamPropertyPhysicalFormat,
+                              &p->original_asbd);
+        CHECK_CA_WARN("could not restore physical stream format");
+    }
 }
 
 static OSStatus hotplug_cb(AudioObjectID id, UInt32 naddr,
                            const AudioObjectPropertyAddress addr[],
-                           void *ctx) {
-    reinit_device(ctx);
-    ao_hotplug_event(ctx);
+                           void *ctx)
+{
+    struct ao *ao = ctx;
+    MP_VERBOSE(ao, "Handling potential hotplug event...\n");
+    reinit_device(ao);
+    ao_hotplug_event(ao);
     return noErr;
 }
 
@@ -412,159 +404,6 @@ static void hotplug_uninit(struct ao *ao)
     }
 }
 
-
-// Channel Mapping functions
-static const int speaker_map[][2] = {
-    { kAudioChannelLabel_Left,                 MP_SPEAKER_ID_FL   },
-    { kAudioChannelLabel_Right,                MP_SPEAKER_ID_FR   },
-    { kAudioChannelLabel_Center,               MP_SPEAKER_ID_FC   },
-    { kAudioChannelLabel_LFEScreen,            MP_SPEAKER_ID_LFE  },
-    { kAudioChannelLabel_LeftSurround,         MP_SPEAKER_ID_BL   },
-    { kAudioChannelLabel_RightSurround,        MP_SPEAKER_ID_BR   },
-    { kAudioChannelLabel_LeftCenter,           MP_SPEAKER_ID_FLC  },
-    { kAudioChannelLabel_RightCenter,          MP_SPEAKER_ID_FRC  },
-    { kAudioChannelLabel_CenterSurround,       MP_SPEAKER_ID_BC   },
-    { kAudioChannelLabel_LeftSurroundDirect,   MP_SPEAKER_ID_SL   },
-    { kAudioChannelLabel_RightSurroundDirect,  MP_SPEAKER_ID_SR   },
-    { kAudioChannelLabel_TopCenterSurround,    MP_SPEAKER_ID_TC   },
-    { kAudioChannelLabel_VerticalHeightLeft,   MP_SPEAKER_ID_TFL  },
-    { kAudioChannelLabel_VerticalHeightCenter, MP_SPEAKER_ID_TFC  },
-    { kAudioChannelLabel_VerticalHeightRight,  MP_SPEAKER_ID_TFR  },
-    { kAudioChannelLabel_TopBackLeft,          MP_SPEAKER_ID_TBL  },
-    { kAudioChannelLabel_TopBackCenter,        MP_SPEAKER_ID_TBC  },
-    { kAudioChannelLabel_TopBackRight,         MP_SPEAKER_ID_TBR  },
-
-    // unofficial extensions
-    { kAudioChannelLabel_RearSurroundLeft,     MP_SPEAKER_ID_SDL  },
-    { kAudioChannelLabel_RearSurroundRight,    MP_SPEAKER_ID_SDR  },
-    { kAudioChannelLabel_LeftWide,             MP_SPEAKER_ID_WL   },
-    { kAudioChannelLabel_RightWide,            MP_SPEAKER_ID_WR   },
-    { kAudioChannelLabel_LFE2,                 MP_SPEAKER_ID_LFE2 },
-
-    { kAudioChannelLabel_HeadphonesLeft,       MP_SPEAKER_ID_DL   },
-    { kAudioChannelLabel_HeadphonesRight,      MP_SPEAKER_ID_DR   },
-
-    { kAudioChannelLabel_Unknown,              MP_SPEAKER_ID_NA  },
-
-    { 0,                                       -1                 },
-};
-
-static int ca_label_to_mp_speaker_id(AudioChannelLabel label)
-{
-    for (int i = 0; speaker_map[i][1] >= 0; i++)
-        if (speaker_map[i][0] == label)
-            return speaker_map[i][1];
-    return -1;
-}
-
-static void ca_log_layout(struct ao *ao, int l, AudioChannelLayout *layout)
-{
-    if (!mp_msg_test(ao->log, l))
-        return;
-
-    AudioChannelDescription *descs = layout->mChannelDescriptions;
-
-    mp_msg(ao->log, l, "layout: tag: <%u>, bitmap: <%u>, "
-                       "descriptions <%u>\n",
-                       (unsigned) layout->mChannelLayoutTag,
-                       (unsigned) layout->mChannelBitmap,
-                       (unsigned) layout->mNumberChannelDescriptions);
-
-    for (int i = 0; i < layout->mNumberChannelDescriptions; i++) {
-        AudioChannelDescription d = descs[i];
-        mp_msg(ao->log, l, " - description %d: label <%u, %u>, "
-            " flags: <%u>, coords: <%f, %f, %f>\n", i,
-            (unsigned) d.mChannelLabel,
-            (unsigned) ca_label_to_mp_speaker_id(d.mChannelLabel),
-            (unsigned) d.mChannelFlags,
-            d.mCoordinates[0],
-            d.mCoordinates[1],
-            d.mCoordinates[2]);
-    }
-}
-
-static AudioChannelLayout *ca_layout_to_custom_layout(
-    struct ao *ao, void *talloc_ctx, AudioChannelLayout *l)
-{
-    AudioChannelLayoutTag tag = l->mChannelLayoutTag;
-    AudioChannelLayout *r;
-    OSStatus err;
-
-    if (tag == kAudioChannelLayoutTag_UseChannelBitmap) {
-        uint32_t psize;
-        err = AudioFormatGetPropertyInfo(
-            kAudioFormatProperty_ChannelLayoutForBitmap,
-            sizeof(uint32_t), &l->mChannelBitmap, &psize);
-        CHECK_CA_ERROR("failed to convert channel bitmap to descriptions (info)");
-        r = talloc_size(NULL, psize);
-        err = AudioFormatGetProperty(
-            kAudioFormatProperty_ChannelLayoutForBitmap,
-            sizeof(uint32_t), &l->mChannelBitmap, &psize, r);
-        CHECK_CA_ERROR("failed to convert channel bitmap to descriptions (get)");
-    } else if (tag != kAudioChannelLayoutTag_UseChannelDescriptions) {
-        uint32_t psize;
-        err = AudioFormatGetPropertyInfo(
-            kAudioFormatProperty_ChannelLayoutForTag,
-            sizeof(AudioChannelLayoutTag), &l->mChannelLayoutTag, &psize);
-        r = talloc_size(NULL, psize);
-        CHECK_CA_ERROR("failed to convert channel tag to descriptions (info)");
-        err = AudioFormatGetProperty(
-            kAudioFormatProperty_ChannelLayoutForTag,
-            sizeof(AudioChannelLayoutTag), &l->mChannelLayoutTag, &psize, r);
-        CHECK_CA_ERROR("failed to convert channel tag to descriptions (get)");
-    } else {
-        r = l;
-    }
-
-    return r;
-coreaudio_error:
-    return NULL;
-}
-
-bool ca_layout_to_mp_chmap(struct ao *ao, AudioChannelLayout *layout,
-                           struct mp_chmap *chmap)
-{
-    void *talloc_ctx = talloc_new(NULL);
-
-    MP_DBG(ao, "input channel layout:\n");
-    ca_log_layout(ao, MSGL_DEBUG, layout);
-
-    AudioChannelLayout *l = ca_layout_to_custom_layout(ao, talloc_ctx, layout);
-    if (!l)
-        goto coreaudio_error;
-
-    MP_VERBOSE(ao, "converted input channel layout:\n");
-    ca_log_layout(ao, MSGL_V, l);
-
-    if (l->mNumberChannelDescriptions > MP_NUM_CHANNELS) {
-        MP_VERBOSE(ao, "layout has too many descriptions (%u, max: %d)\n",
-                   (unsigned) l->mNumberChannelDescriptions, MP_NUM_CHANNELS);
-        return false;
-    }
-
-    int next_na = MP_SPEAKER_ID_NA;
-    for (int n = 0; n < l->mNumberChannelDescriptions; n++) {
-        AudioChannelLabel label = l->mChannelDescriptions[n].mChannelLabel;
-        int speaker = ca_label_to_mp_speaker_id(label);
-        if (speaker < 0) {
-            MP_VERBOSE(ao, "channel label=%u unusable to build channel "
-                           "bitmap, skipping layout\n", (unsigned) label);
-            goto coreaudio_error;
-        } else {
-            chmap->speaker[n] = speaker;
-            chmap->num = n + 1;
-        }
-    }
-
-    talloc_free(talloc_ctx);
-    return chmap->num > 0;
-coreaudio_error:
-    MP_VERBOSE(ao, "converted input channel layout (failed):\n");
-    ca_log_layout(ao, MSGL_V, layout);
-    talloc_free(talloc_ctx);
-    return false;
-}
-
 #define OPT_BASE_STRUCT struct priv
 
 const struct ao_driver audio_out_coreaudio = {
@@ -579,4 +418,9 @@ const struct ao_driver audio_out_coreaudio = {
     .hotplug_uninit = hotplug_uninit,
     .list_devs      = ca_get_device_list,
     .priv_size      = sizeof(struct priv),
+    .options = (const struct m_option[]){
+        OPT_FLAG("change-physical-format", change_physical_format, 0),
+        OPT_FLAG("exclusive", exclusive, 0),
+        {0}
+    },
 };
