@@ -4,6 +4,7 @@
 #include <stdbool.h>
 #include <stdint.h>
 #include "m_option.h"
+#include "common/common.h"
 
 typedef struct mp_vo_opts {
     struct m_obj_settings *video_driver_list, *vo_defs;
@@ -30,6 +31,7 @@ typedef struct mp_vo_opts {
     struct m_geometry autofit;
     struct m_geometry autofit_larger;
     struct m_geometry autofit_smaller;
+    float window_scale;
 
     int keepaspect;
     int keepaspect_window;
@@ -40,7 +42,10 @@ typedef struct mp_vo_opts {
     float monitor_pixel_aspect;
     int force_window_position;
 
+    // vo_wayland, vo_drm
     struct sws_opts *sws_opts;
+    // vo_opengl, vo_opengl_cb
+    int hwdec_preload_api;
 } mp_vo_opts;
 
 struct mp_cache_opts {
@@ -48,6 +53,7 @@ struct mp_cache_opts {
     int def_size;
     int initial;
     int seek_min;
+    int back_buffer;
     char *file;
     int file_max;
 };
@@ -68,6 +74,7 @@ typedef struct MPOpts {
     int lua_load_osc;
     int lua_load_ytdl;
     char *lua_ytdl_format;
+    char **lua_ytdl_raw_options;
 
     int auto_load_scripts;
 
@@ -101,16 +108,12 @@ typedef struct MPOpts {
     int cursor_autohide_delay;
     int cursor_autohide_fs;
 
-    int requested_colorspace;
-    int requested_input_range;
-    int requested_output_range;
-    int requested_primaries;
-
     int video_rotate;
     int video_stereo_mode;
 
     char *audio_decoders;
     char *video_decoders;
+    char *audio_spdif;
 
     int osd_level;
     int osd_duration;
@@ -141,6 +144,9 @@ typedef struct MPOpts {
     int correct_pts;
     int user_pts_assoc_mode;
     int initial_audio_sync;
+    int video_sync;
+    double sync_max_video_change;
+    double sync_max_audio_change;
     int hr_seek;
     float hr_seek_demuxer_offset;
     int hr_seek_framedrop;
@@ -161,6 +167,7 @@ typedef struct MPOpts {
     float heartbeat_interval;
     int player_idle_mode;
     int consolecontrols;
+    int playlist_pos;
     struct m_rel_time play_start;
     struct m_rel_time play_end;
     struct m_rel_time play_length;
@@ -173,15 +180,9 @@ typedef struct MPOpts {
     int ignore_path_in_watch_later_config;
     int pause;
     int keep_open;
-    int audio_id;
-    int video_id;
-    int sub_id;
-    int sub2_id;
-    int audio_id_ff;
-    int video_id_ff;
-    int sub_id_ff;
-    char **audio_lang;
-    char **sub_lang;
+    int stream_id[2][STREAM_TYPE_COUNT];
+    int stream_id_ff[STREAM_TYPE_COUNT];
+    char **stream_lang[STREAM_TYPE_COUNT];
     int audio_display;
     char **display_tags;
     int sub_visibility;
@@ -191,27 +192,27 @@ typedef struct MPOpts {
     float sub_speed;
     int forced_subs_only;
     int stretch_dvd_subs;
+    int stretch_image_subs;
 
     int sub_fix_timing;
     char *sub_cp;
 
     char **audio_files;
     char *demuxer_name;
+    int demuxer_max_packs;
+    int demuxer_max_bytes;
     int demuxer_thread;
-    int demuxer_min_packs;
-    int demuxer_min_bytes;
     double demuxer_min_secs;
     char *audio_demuxer_name;
     char *sub_demuxer_name;
-    int mkv_subtitle_preroll;
-    double mkv_subtitle_preroll_secs;
-    int mkv_probe_duration;
+    int force_seekable;
 
     double demuxer_min_secs_cache;
     int cache_pausing;
 
     struct image_writer_opts *screenshot_image_opts;
     char *screenshot_template;
+    char *screenshot_directory;
 
     double force_fps;
     int index_mode;
@@ -226,6 +227,7 @@ typedef struct MPOpts {
     struct m_obj_settings *af_settings, *af_defs;
     int deinterlace;
     float movie_aspect;
+    int aspect_method;
     int field_dominance;
     char **sub_name;
     char **sub_paths;
@@ -241,6 +243,7 @@ typedef struct MPOpts {
     int osd_scale_by_window;
     int sub_scale_by_window;
     int sub_scale_with_window;
+    int ass_scale_with_window;
     struct osd_style_opts *osd_style;
     struct osd_style_opts *sub_text_style;
     float sub_scale;
@@ -249,6 +252,7 @@ typedef struct MPOpts {
     int ass_enabled;
     float ass_line_spacing;
     int ass_use_margins;
+    int sub_use_margins;
     int ass_vsfilter_aspect_compat;
     int ass_vsfilter_color_compat;
     int ass_vsfilter_blur_compat;
@@ -265,8 +269,6 @@ typedef struct MPOpts {
 
     int w32_priority;
 
-    int slave_mode;
-
     int network_cookies_enabled;
     char *network_cookies_file;
     char *network_useragent;
@@ -296,6 +298,7 @@ typedef struct MPOpts {
     struct demux_rawaudio_opts *demux_rawaudio;
     struct demux_rawvideo_opts *demux_rawvideo;
     struct demux_lavf_opts *demux_lavf;
+    struct demux_mkv_opts *demux_mkv;
 
     struct vd_lavc_params *vd_lavc_params;
     struct ad_lavc_params *ad_lavc_params;
