@@ -20,6 +20,7 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <fcntl.h>
 #include <errno.h>
 #include <signal.h>
 
@@ -29,8 +30,6 @@
 #include "common/common.h"
 #include "stream/stream.h"
 
-// Normally, this must be declared manually, but glibc is retarded
-// resulting in a warning.
 extern char **environ;
 
 // A silly helper: automatically skips entries with negative FDs
@@ -63,17 +62,26 @@ int mp_subprocess(char **args, struct mp_cancel *cancel, void *ctx,
     int status = -1;
     int p_stdout[2] = {-1, -1};
     int p_stderr[2] = {-1, -1};
+    int devnull = -1;
     pid_t pid = -1;
+    bool spawned = false;
+    bool killed_by_us = false;
 
     if (on_stdout && mp_make_cloexec_pipe(p_stdout) < 0)
         goto done;
     if (on_stderr && mp_make_cloexec_pipe(p_stderr) < 0)
         goto done;
 
+    devnull = open("/dev/null", O_RDONLY | O_CLOEXEC);
+    if (devnull < 0)
+        goto done;
+
     if (posix_spawn_file_actions_init(&fa))
         goto done;
     fa_destroy = true;
-    // redirect stdout and stderr
+    // redirect stdin/stdout/stderr
+    if (posix_spawn_file_actions_adddup2(&fa, devnull, 0))
+        goto done;
     if (p_stdout[1] >= 0 && posix_spawn_file_actions_adddup2(&fa, p_stdout[1], 1))
         goto done;
     if (p_stderr[1] >= 0 && posix_spawn_file_actions_adddup2(&fa, p_stderr[1], 2))
@@ -83,11 +91,14 @@ int mp_subprocess(char **args, struct mp_cancel *cancel, void *ctx,
         pid = -1;
         goto done;
     }
+    spawned = true;
 
     close(p_stdout[1]);
     p_stdout[1] = -1;
     close(p_stderr[1]);
     p_stderr[1] = -1;
+    close(devnull);
+    devnull = -1;
 
     int *read_fds[2] = {&p_stdout[0], &p_stderr[0]};
     subprocess_read_cb read_cbs[2] = {on_stdout, on_stderr};
@@ -116,6 +127,7 @@ int mp_subprocess(char **args, struct mp_cancel *cancel, void *ctx,
         }
         if (fds[2].revents) {
             kill(pid, SIGKILL);
+            killed_by_us = true;
             break;
         }
     }
@@ -133,13 +145,17 @@ done:
     close(p_stdout[1]);
     close(p_stderr[0]);
     close(p_stderr[1]);
+    close(devnull);
 
-    if (WIFEXITED(status) && WEXITSTATUS(status) != 127) {
+    if (!spawned || (WIFEXITED(status) && WEXITSTATUS(status) == 127)) {
+        *error = "init";
+        status = -1;
+    } else if (WIFEXITED(status)) {
         *error = NULL;
         status = WEXITSTATUS(status);
     } else {
-        *error = WEXITSTATUS(status) == 127 ? "init" : "killed";
-        status = -1;
+        *error = "killed";
+        status = killed_by_us ? MP_SUBPROCESS_EKILLED_BY_US : -1;
     }
 
     return status;
