@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with mpv; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stddef.h>
@@ -43,7 +42,7 @@ struct sub_cache {
 };
 
 struct part {
-    int bitmap_pos_id;
+    int change_id;
     int imgfmt;
     enum mp_csp colorspace;
     enum mp_csp_levels levels;
@@ -289,17 +288,16 @@ static void draw_ass(struct mp_draw_sub_cache *cache, struct mp_rect bb,
                      struct mp_image *temp, int bits, struct sub_bitmaps *sbs)
 {
     struct mp_csp_params cspar = MP_CSP_PARAMS_DEFAULTS;
-    cspar.colorspace.format = temp->params.colorspace;
-    cspar.colorspace.levels_in = temp->params.colorlevels;
-    cspar.colorspace.levels_out = MP_CSP_LEVELS_PC; // RGB (libass.color)
+    mp_csp_set_image_params(&cspar, &temp->params);
+    cspar.levels_out = MP_CSP_LEVELS_PC; // RGB (libass.color)
     cspar.int_bits_in = bits;
     cspar.int_bits_out = 8;
 
-    float yuv2rgb[3][4], rgb2yuv[3][4];
-    bool need_conv = temp->flags & MP_IMGFLAG_YUV;
+    struct mp_cmat yuv2rgb, rgb2yuv;
+    bool need_conv = temp->fmt.flags & MP_IMGFLAG_YUV;
     if (need_conv) {
-        mp_get_yuv2rgb_coeffs(&cspar, yuv2rgb);
-        mp_invert_yuv2rgb(rgb2yuv, yuv2rgb);
+        mp_get_yuv2rgb_coeffs(&cspar, &yuv2rgb);
+        mp_invert_yuv2rgb(&rgb2yuv, &yuv2rgb);
     }
 
     for (int i = 0; i < sbs->num_parts; ++i) {
@@ -316,7 +314,7 @@ static void draw_ass(struct mp_draw_sub_cache *cache, struct mp_rect bb,
         int a = 255 - (sb->libass.color & 0xFF);
         int color_yuv[3] = {r, g, b};
         if (need_conv) {
-            mp_map_int_color(rgb2yuv, bits, color_yuv);
+            mp_map_int_color(&rgb2yuv, bits, color_yuv);
         } else {
             color_yuv[0] = g;
             color_yuv[1] = b;
@@ -335,13 +333,13 @@ static void draw_ass(struct mp_draw_sub_cache *cache, struct mp_rect bb,
 static void get_swscale_alignment(const struct mp_image *img, int *out_xstep,
                                   int *out_ystep)
 {
-    int sx = (1 << img->chroma_x_shift);
-    int sy = (1 << img->chroma_y_shift);
+    int sx = (1 << img->fmt.chroma_xs);
+    int sy = (1 << img->fmt.chroma_ys);
 
     for (int p = 0; p < img->num_planes; ++p) {
         int bits = img->fmt.bpp[p];
         // the * 2 fixes problems with writing past the destination width
-        while (((sx >> img->chroma_x_shift) * bits) % (SWS_MIN_BYTE_ALIGN * 8 * 2))
+        while (((sx >> img->fmt.chroma_xs) * bits) % (SWS_MIN_BYTE_ALIGN * 8 * 2))
             sx *= 2;
     }
 
@@ -400,7 +398,7 @@ static struct part *get_cache(struct mp_draw_sub_cache *cache,
     if (use_cache) {
         part = cache->parts[sbs->render_index];
         if (part) {
-            if (part->bitmap_pos_id != sbs->bitmap_pos_id
+            if (part->change_id != sbs->change_id
                 || part->imgfmt != format->imgfmt
                 || part->colorspace != format->params.colorspace
                 || part->levels != format->params.colorlevels)
@@ -412,7 +410,7 @@ static struct part *get_cache(struct mp_draw_sub_cache *cache,
         if (!part) {
             part = talloc(cache, struct part);
             *part = (struct part) {
-                .bitmap_pos_id = sbs->bitmap_pos_id,
+                .change_id = sbs->change_id,
                 .num_imgs = sbs->num_parts,
                 .imgfmt = format->imgfmt,
                 .levels = format->params.colorlevels,
@@ -471,7 +469,7 @@ static struct mp_image *chroma_up(struct mp_draw_sub_cache *cache, int imgfmt,
 
     // The temp image is always YUV, but src not necessarily.
     // Reduce amount of conversions in YUV case (upsampling/shifting only)
-    if (src->flags & MP_IMGFLAG_YUV) {
+    if (src->fmt.flags & MP_IMGFLAG_YUV) {
         temp->params.colorspace = src->params.colorspace;
         temp->params.colorlevels = src->params.colorlevels;
     }
@@ -482,9 +480,9 @@ static struct mp_image *chroma_up(struct mp_draw_sub_cache *cache, int imgfmt,
         // The whole point is not having swscale copy the Y plane
         struct mp_image t_dst = *temp;
         mp_image_setfmt(&t_dst, IMGFMT_Y8);
-        mp_image_set_size(&t_dst, temp->chroma_width, temp->chroma_height);
+        mp_image_set_size(&t_dst, temp->w, temp->h);
         struct mp_image t_src = t_dst;
-        mp_image_set_size(&t_src, src->chroma_width, src->chroma_height);
+        mp_image_set_size(&t_src, src->w >> 1, src->h >> 1);
         for (int c = 0; c < 2; c++) {
             t_dst.planes[0] = temp->planes[1 + c];
             t_dst.stride[0] = temp->stride[1 + c];
@@ -512,10 +510,9 @@ static void chroma_down(struct mp_image *old_src, struct mp_image *temp)
             assert(temp->planes[0] == old_src->planes[0]);
             struct mp_image t_dst = *temp;
             mp_image_setfmt(&t_dst, IMGFMT_Y8);
-            mp_image_set_size(&t_dst, old_src->chroma_width,
-                              old_src->chroma_height);
+            mp_image_set_size(&t_dst, old_src->w >> 1, old_src->h >> 1);
             struct mp_image t_src = t_dst;
-            mp_image_set_size(&t_src, temp->chroma_width, temp->chroma_height);
+            mp_image_set_size(&t_src, temp->w, temp->h);
             for (int c = 0; c < 2; c++) {
                 t_dst.planes[0] = old_src->planes[1 + c];
                 t_dst.stride[0] = old_src->stride[1 + c];
