@@ -1,19 +1,18 @@
 /*
- * This file is part of MPlayer.
+ * This file is part of mpv.
  *
- * MPlayer is free software; you can redistribute it and/or modify
+ * mpv is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * MPlayer is distributed in the hope that it will be useful,
+ * mpv is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with MPlayer; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stdio.h>
@@ -41,37 +40,62 @@
 #include "video/mp_image.h"
 #include "video/mp_image_pool.h"
 
-static const struct osd_style_opts osd_style_opts_def = {
-    .font = "sans-serif",
-    .font_size = 55,
-    .color = {255, 255, 255, 255},
-    .border_color = {0, 0, 0, 255},
-    .shadow_color = {240, 240, 240, 128},
-    .border_size = 3,
-    .shadow_offset = 0,
-    .margin_x = 25,
-    .margin_y = 22,
+#define OPT_BASE_STRUCT struct osd_style_opts
+static const m_option_t style_opts[] = {
+    OPT_STRING("font", font, 0),
+    OPT_FLOATRANGE("font-size", font_size, 0, 1, 9000),
+    OPT_COLOR("color", color, 0),
+    OPT_COLOR("border-color", border_color, 0),
+    OPT_COLOR("shadow-color", shadow_color, 0),
+    OPT_COLOR("back-color", back_color, 0),
+    OPT_FLOATRANGE("border-size", border_size, 0, 0, 10),
+    OPT_FLOATRANGE("shadow-offset", shadow_offset, 0, 0, 10),
+    OPT_FLOATRANGE("spacing", spacing, 0, -10, 10),
+    OPT_INTRANGE("margin-x", margin_x, 0, 0, 300),
+    OPT_INTRANGE("margin-y", margin_y, 0, 0, 600),
+    OPT_CHOICE("align-x", align_x, 0,
+               ({"left", -1}, {"center", 0}, {"right", +1})),
+    OPT_CHOICE("align-y", align_y, 0,
+               ({"top", -1}, {"center", 0}, {"bottom", +1})),
+    OPT_FLOATRANGE("blur", blur, 0, 0, 20),
+    OPT_FLAG("bold", bold, 0),
+    {0}
 };
 
-#define OPT_BASE_STRUCT struct osd_style_opts
 const struct m_sub_options osd_style_conf = {
-    .opts = (const m_option_t[]) {
-        OPT_STRING("font", font, 0),
-        OPT_FLOATRANGE("font-size", font_size, 0, 1, 9000),
-        OPT_COLOR("color", color, 0),
-        OPT_COLOR("border-color", border_color, 0),
-        OPT_COLOR("shadow-color", shadow_color, 0),
-        OPT_COLOR("back-color", back_color, 0),
-        OPT_FLOATRANGE("border-size", border_size, 0, 0, 10),
-        OPT_FLOATRANGE("shadow-offset", shadow_offset, 0, 0, 10),
-        OPT_FLOATRANGE("spacing", spacing, 0, -10, 10),
-        OPT_INTRANGE("margin-x", margin_x, 0, 0, 300),
-        OPT_INTRANGE("margin-y", margin_y, 0, 0, 600),
-        OPT_FLOATRANGE("blur", blur, 0, 0, 20),
-        {0}
-    },
+    .opts = style_opts,
     .size = sizeof(struct osd_style_opts),
-    .defaults = &osd_style_opts_def,
+    .defaults = &(const struct osd_style_opts){
+        .font = "sans-serif",
+        .font_size = 55,
+        .color = {255, 255, 255, 255},
+        .border_color = {0, 0, 0, 255},
+        .shadow_color = {240, 240, 240, 128},
+        .border_size = 3,
+        .shadow_offset = 0,
+        .margin_x = 25,
+        .margin_y = 22,
+        .align_x = -1,
+        .align_y = -1,
+    },
+};
+
+const struct m_sub_options sub_style_conf = {
+    .opts = style_opts,
+    .size = sizeof(struct osd_style_opts),
+    .defaults = &(const struct osd_style_opts){
+        .font = "sans-serif",
+        .font_size = 55,
+        .color = {255, 255, 255, 255},
+        .border_color = {0, 0, 0, 255},
+        .shadow_color = {240, 240, 240, 128},
+        .border_size = 3,
+        .shadow_offset = 0,
+        .margin_x = 25,
+        .margin_y = 22,
+        .align_x = 0,
+        .align_y = 1,
+    },
 };
 
 static bool osd_res_equals(struct mp_osd_res a, struct mp_osd_res b)
@@ -200,14 +224,6 @@ void osd_set_external2(struct osd_state *osd, struct sub_bitmaps *imgs)
     pthread_mutex_unlock(&osd->lock);
 }
 
-void osd_set_nav_highlight(struct osd_state *osd, void *priv)
-{
-    pthread_mutex_lock(&osd->lock);
-    osd->objs[OSDTYPE_NAV_HIGHLIGHT]->highlight_priv = priv;
-    osd_changed_unlocked(osd, OSDTYPE_NAV_HIGHLIGHT);
-    pthread_mutex_unlock(&osd->lock);
-}
-
 static void render_object(struct osd_state *osd, struct osd_object *obj,
                           struct mp_osd_res res, double video_pts,
                           const bool sub_formats[SUBBITMAP_COUNT],
@@ -239,38 +255,29 @@ static void render_object(struct osd_state *osd, struct osd_object *obj,
     } else if (obj->type == OSDTYPE_EXTERNAL2) {
         if (obj->external2 && obj->external2->format) {
             *out_imgs = *obj->external2;
-            obj->external2->bitmap_id = obj->external2->bitmap_pos_id = 0;
+            obj->external2->change_id = 0;
         }
-    } else if (obj->type == OSDTYPE_NAV_HIGHLIGHT) {
-        if (obj->highlight_priv)
-            mp_nav_get_highlight(obj->highlight_priv, obj->vo_res, out_imgs);
     } else {
         osd_object_get_bitmaps(osd, obj, out_imgs);
     }
 
-    if (obj->force_redraw) {
-        out_imgs->bitmap_id++;
-        out_imgs->bitmap_pos_id++;
-    }
+    if (obj->force_redraw)
+        out_imgs->change_id++;
 
     obj->force_redraw = false;
-    obj->vo_bitmap_id += out_imgs->bitmap_id;
-    obj->vo_bitmap_pos_id += out_imgs->bitmap_pos_id;
+    obj->vo_change_id += out_imgs->change_id;
 
     if (out_imgs->num_parts == 0)
         return;
 
-    if (obj->cached.bitmap_id == obj->vo_bitmap_id
-        && obj->cached.bitmap_pos_id == obj->vo_bitmap_pos_id
-        && formats[obj->cached.format])
+    if (obj->cached.change_id == obj->vo_change_id && formats[obj->cached.format])
     {
         *out_imgs = obj->cached;
         return;
     }
 
     out_imgs->render_index = obj->type;
-    out_imgs->bitmap_id = obj->vo_bitmap_id;
-    out_imgs->bitmap_pos_id = obj->vo_bitmap_pos_id;
+    out_imgs->change_id = obj->vo_change_id;
 
     if (formats[out_imgs->format])
         return;
@@ -314,6 +321,8 @@ void osd_draw(struct osd_state *osd, struct mp_osd_res res,
             continue;
         if ((draw_flags & OSD_DRAW_SUB_ONLY) && !obj->is_sub)
             continue;
+        if ((draw_flags & OSD_DRAW_OSD_ONLY) && obj->is_sub)
+            continue;
 
         if (obj->sub_state.dec_sub)
             sub_lock(obj->sub_state.dec_sub);
