@@ -6,8 +6,8 @@
 #include "talloc.h"
 #include "osd.h"
 
-const char *osd_ass_0 = "";
-const char *osd_ass_1 = "";
+const char *const osd_ass_0 = "";
+const char *const osd_ass_1 = "";
 
 void osd_init_backend(struct osd_state *osd)
 {
