@@ -28,16 +28,12 @@ struct osd_object {
     // OSDTYPE_EXTERNAL2
     struct sub_bitmaps *external2;
 
-    // OSDTYPE_NAV_HIGHLIGHT
-    void *highlight_priv;
-
     // caches for OSD conversion (internal to render_object())
     struct osd_conv_cache *cache[OSD_CONV_CACHE_MAX];
     struct sub_bitmaps cached;
 
     // VO cache state
-    int vo_bitmap_id;
-    int vo_bitmap_pos_id;
+    int vo_change_id;
     struct mp_osd_res vo_res;
 
     // Internally used by osd_libass.c
