@@ -1,18 +1,18 @@
 /*
- * This file is part of mplayer2.
+ * This file is part of mpv.
  *
- * mplayer2 is free software; you can redistribute it and/or modify
+ * mpv is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * mplayer2 is distributed in the hope that it will be useful,
+ * mpv is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with mplayer2.  If not, see <http://www.gnu.org/licenses/>.
+ * with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -28,7 +28,6 @@
 #include "common/av_common.h"
 #include "options/options.h"
 #include "video/mp_image.h"
-#include "video/csputils.h"
 #include "sd.h"
 #include "dec_sub.h"
 
@@ -98,35 +97,6 @@ static void get_resolution(struct sd *sd, int wh[2])
     }
 }
 
-static void set_mp4_vobsub_idx(AVCodecContext *avctx, char *src, int w, int h)
-{
-    char pal_s[128];
-    int pal_s_pos = 0;
-    for (int i = 0; i < 16; i++) {
-        unsigned int e = AV_RB32(src + i * 4);
-
-        // lavc doesn't accept YUV palette - "does god hate me?"
-        struct mp_csp_params csp = MP_CSP_PARAMS_DEFAULTS;
-        csp.int_bits_in = 8;
-        csp.int_bits_out = 8;
-        struct mp_cmat cmatrix;
-        mp_get_yuv2rgb_coeffs(&csp, &cmatrix);
-        int c[3] = {(e >> 16) & 0xff, (e >> 8) & 0xff, e & 0xff};
-        mp_map_int_color(&cmatrix, 8, c);
-        e = (c[2] << 16) | (c[1] << 8) | c[0];
-
-        snprintf(pal_s + pal_s_pos, sizeof(pal_s) - pal_s_pos, "%06x%s", e,
-                 i != 15 ? ", " : "");
-        pal_s_pos = strlen(pal_s);
-        if (pal_s_pos >= sizeof(pal_s))
-            break;
-    }
-
-    char buf[256] = "";
-    snprintf(buf, sizeof(buf), "size: %dx%d\npalette: %s\n", w, h, pal_s);
-    mp_lavc_set_extradata(avctx, buf, strlen(buf));
-}
-
 static int init(struct sd *sd)
 {
     struct sd_lavc_priv *priv = talloc_zero(NULL, struct sd_lavc_priv);
@@ -139,11 +109,6 @@ static int init(struct sd *sd)
     if (!ctx)
         goto error;
     mp_lavc_set_extradata(ctx, sd->extradata, sd->extradata_len);
-    if (sd->extradata_len == 64 && sd->sub_stream_w && sd->sub_stream_h &&
-        cid == AV_CODEC_ID_DVD_SUBTITLE)
-    {
-        set_mp4_vobsub_idx(ctx, sd->extradata, sd->sub_stream_w, sd->sub_stream_h);
-    }
     if (avcodec_open2(ctx, sub_codec, NULL) < 0)
         goto error;
     priv->avctx = ctx;
@@ -291,7 +256,7 @@ static void get_bitmaps(struct sd *sd, struct mp_osd_res d, double pts,
     res->parts = priv->outbitmaps;
     res->num_parts = current->count;
     if (priv->displayed_id != current->id)
-        res->bitmap_id = ++res->bitmap_pos_id;
+        res->change_id++;
     priv->displayed_id = current->id;
     res->format = SUBBITMAP_INDEXED;
 
@@ -307,6 +272,8 @@ static void get_bitmaps(struct sd *sd, struct mp_osd_res d, double pts,
     }
     if (priv->avctx->codec_id == AV_CODEC_ID_HDMV_PGS_SUBTITLE)
         video_par = -1;
+    if (opts->stretch_image_subs)
+        d.ml = d.mr = d.mt = d.mb = 0;
     int insize[2];
     get_resolution(sd, insize);
     osd_rescale_bitmaps(res, insize[0], insize[1], d, video_par);
