@@ -36,6 +36,7 @@
 #include "video/fmt-conversion.h"
 #include "video/mp_image_pool.h"
 #include "video/hwdec.h"
+#include "video/d3d.h"
 #include "gpu_memcpy_sse4.h"
 
 // A minor evil.
@@ -55,6 +56,7 @@ DEFINE_GUID(DXVA2_ModeH264_F,         0x1b81be69, 0xa0c7,0x11d3,0xb9,0x84,0x00,0
 DEFINE_GUID(DXVADDI_Intel_ModeH264_E, 0x604F8E68, 0x4951,0x4C54,0x88,0xFE,0xAB,0xD2,0x5C,0x15,0xB3,0xD6);
 DEFINE_GUID(DXVA2_ModeVC1_D,          0x1b81beA3, 0xa0c7,0x11d3,0xb9,0x84,0x00,0xc0,0x4f,0x2e,0x73,0xc5);
 DEFINE_GUID(DXVA2_ModeVC1_D2010,      0x1b81beA4, 0xa0c7,0x11d3,0xb9,0x84,0x00,0xc0,0x4f,0x2e,0x73,0xc5);
+DEFINE_GUID(DXVA2_ModeHEVC_VLD_Main,  0x5b11d51b, 0x2f4c,0x4452,0xbc,0xc3,0x09,0xf2,0xa1,0x16,0x0c,0xc0);
 DEFINE_GUID(DXVA2_NoEncrypt,          0x1b81beD0, 0xa0c7,0x11d3,0xb9,0x84,0x00,0xc0,0x4f,0x2e,0x73,0xc5);
 DEFINE_GUID(GUID_NULL,                0x00000000, 0x0000,0x0000,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00);
 
@@ -83,6 +85,8 @@ static const dxva2_mode dxva2_modes[] = {
     { &DXVA2_ModeVC1_D,          AV_CODEC_ID_VC1  },
     { &DXVA2_ModeVC1_D,          AV_CODEC_ID_WMV3 },
 
+    { &DXVA2_ModeHEVC_VLD_Main,  AV_CODEC_ID_HEVC },
+
     { NULL,                      0 },
 };
 
@@ -201,14 +205,11 @@ static void dxva2_release_img(void *ptr)
     av_free(w);
 }
 
-static struct mp_image *dxva2_allocate_image(struct lavc_ctx *s, int fmt,
+static struct mp_image *dxva2_allocate_image(struct lavc_ctx *s,
                                              int img_w, int img_h)
 {
     DXVA2Context *ctx = s->hwdec_priv;
 
-    if (fmt != IMGFMT_DXVA2)
-        return NULL;
-
     int i, old_unused = -1;
     for (i = 0; i < ctx->num_surfaces; i++) {
         surface_info *info = &ctx->surface_infos[i];
@@ -307,8 +308,11 @@ static struct mp_image *dxva2_retrieve_image(struct lavc_ctx *s,
 
     IDirect3DSurface9_GetDesc(surface, &surfaceDesc);
 
+    if (surfaceDesc.Width < img->w || surfaceDesc.Height < img->h)
+        return img;
+
     struct mp_image *sw_img =
-        mp_image_pool_get(ctx->sw_pool, IMGFMT_NV12, img->w, img->h);
+        mp_image_pool_get(ctx->sw_pool, IMGFMT_NV12, surfaceDesc.Width, surfaceDesc.Height);
 
     if (!sw_img)
         return img;
@@ -321,6 +325,7 @@ static struct mp_image *dxva2_retrieve_image(struct lavc_ctx *s,
     }
 
     ctx->copy_nv12(sw_img, LockedRect.pBits, LockedRect.Pitch, surfaceDesc.Height);
+    mp_image_set_size(sw_img, img->w, img->h);
     mp_image_copy_attributes(sw_img, img);
 
     IDirect3DSurface9_UnlockRect(surface);
@@ -329,58 +334,35 @@ static struct mp_image *dxva2_retrieve_image(struct lavc_ctx *s,
     return sw_img;
 }
 
-static int dxva2_init(struct lavc_ctx *s)
+static int create_device(struct lavc_ctx *s)
 {
-    DXVA2Context *ctx;
+    DXVA2Context *ctx = s->hwdec_priv;
     pDirect3DCreate9      *createD3D = NULL;
-    pCreateDeviceManager9 *createDeviceManager = NULL;
     HRESULT hr;
     D3DPRESENT_PARAMETERS d3dpp = {0};
     D3DDISPLAYMODE        d3ddm;
-    unsigned resetToken = 0;
     UINT adapter = D3DADAPTER_DEFAULT;
 
-    ctx = talloc_zero(NULL, DXVA2Context);
-    if (!ctx)
-        return -1;
-    s->hwdec_priv = ctx;
-
-    ctx->log = mp_log_new(s, s->log, "dxva2");
-    ctx->sw_pool = talloc_steal(ctx, mp_image_pool_new(17));
-
-    if (av_get_cpu_flags() & AV_CPU_FLAG_SSE4) {
-        // Use a memcpy implementation optimised for copying from GPU memory
-        MP_DBG(ctx, "Using SSE4 memcpy\n");
-        ctx->copy_nv12 = copy_nv12_gpu_sse4;
-    } else {
-        // Use the CRT memcpy. This can be slower than software decoding.
-        MP_WARN(ctx, "Using fallback memcpy (slow)\n");
-        ctx->copy_nv12 = copy_nv12_fallback;
+    if (s->hwdec_info && s->hwdec_info->hwctx && s->hwdec_info->hwctx->d3d_ctx) {
+        ctx->d3d9device = s->hwdec_info->hwctx->d3d_ctx->d3d9_device;
+        if (ctx->d3d9device) {
+            IDirect3D9_AddRef(ctx->d3d9device);
+            MP_VERBOSE(ctx, "Using VO-supplied device %p.\n", ctx->d3d9device);
+            return 0;
+        }
     }
 
-    ctx->deviceHandle = INVALID_HANDLE_VALUE;
-
     ctx->d3dlib = LoadLibrary(L"d3d9.dll");
     if (!ctx->d3dlib) {
         MP_ERR(ctx, "Failed to load D3D9 library\n");
         goto fail;
     }
-    ctx->dxva2lib = LoadLibrary(L"dxva2.dll");
-    if (!ctx->dxva2lib) {
-        MP_ERR(ctx, "Failed to load DXVA2 library\n");
-        goto fail;
-    }
 
     createD3D = (pDirect3DCreate9 *)GetProcAddress(ctx->d3dlib, "Direct3DCreate9");
     if (!createD3D) {
         MP_ERR(ctx, "Failed to locate Direct3DCreate9\n");
         goto fail;
     }
-    createDeviceManager = (pCreateDeviceManager9 *)GetProcAddress(ctx->dxva2lib, "DXVA2CreateDirect3DDeviceManager9");
-    if (!createDeviceManager) {
-        MP_ERR(ctx, "Failed to locate DXVA2CreateDirect3DDeviceManager9\n");
-        goto fail;
-    }
 
     ctx->d3d9 = createD3D(D3D_SDK_VERSION);
     if (!ctx->d3d9) {
@@ -405,6 +387,54 @@ static int dxva2_init(struct lavc_ctx *s)
         goto fail;
     }
 
+    return 0;
+
+fail:
+    return -1;
+}
+
+static int dxva2_init(struct lavc_ctx *s)
+{
+    DXVA2Context *ctx;
+    pCreateDeviceManager9 *createDeviceManager = NULL;
+    HRESULT hr;
+    unsigned resetToken = 0;
+
+    ctx = talloc_zero(NULL, DXVA2Context);
+    if (!ctx)
+        return -1;
+    s->hwdec_priv = ctx;
+
+    ctx->log = mp_log_new(s, s->log, "dxva2");
+    ctx->sw_pool = talloc_steal(ctx, mp_image_pool_new(17));
+
+    if (av_get_cpu_flags() & AV_CPU_FLAG_SSE4) {
+        // Use a memcpy implementation optimised for copying from GPU memory
+        MP_DBG(ctx, "Using SSE4 memcpy\n");
+        ctx->copy_nv12 = copy_nv12_gpu_sse4;
+    } else {
+        // Use the CRT memcpy. This can be slower than software decoding.
+        MP_WARN(ctx, "Using fallback memcpy (slow)\n");
+        ctx->copy_nv12 = copy_nv12_fallback;
+    }
+
+    ctx->deviceHandle = INVALID_HANDLE_VALUE;
+
+    ctx->dxva2lib = LoadLibrary(L"dxva2.dll");
+    if (!ctx->dxva2lib) {
+        MP_ERR(ctx, "Failed to load DXVA2 library\n");
+        goto fail;
+    }
+
+    if (create_device(s) < 0)
+        goto fail;
+
+    createDeviceManager = (pCreateDeviceManager9 *)GetProcAddress(ctx->dxva2lib, "DXVA2CreateDirect3DDeviceManager9");
+    if (!createDeviceManager) {
+        MP_ERR(ctx, "Failed to locate DXVA2CreateDirect3DDeviceManager9\n");
+        goto fail;
+    }
+
     hr = createDeviceManager(&resetToken, &ctx->d3d9devmgr);
     if (FAILED(hr)) {
         MP_ERR(ctx, "Failed to create Direct3D device manager\n");
@@ -558,6 +588,10 @@ static int dxva2_create_decoder(struct lavc_ctx *s, int w, int h,
        but it causes issues for H.264 on certain AMD GPUs..... */
     if (codec_id == AV_CODEC_ID_MPEG2VIDEO)
         surface_alignment = 32;
+    /* the HEVC DXVA2 spec asks for 128 pixel aligned surfaces to ensure
+       all coding features have enough room to work with */
+    else if  (codec_id == AV_CODEC_ID_HEVC)
+        surface_alignment = 128;
     else
         surface_alignment = 16;
 
@@ -565,7 +599,7 @@ static int dxva2_create_decoder(struct lavc_ctx *s, int w, int h,
     ctx->num_surfaces = 4;
 
     /* add surfaces based on number of possible refs */
-    if (codec_id == AV_CODEC_ID_H264)
+    if (codec_id == AV_CODEC_ID_H264 || codec_id == AV_CODEC_ID_HEVC)
         ctx->num_surfaces += 16;
     else
         ctx->num_surfaces += 2;
@@ -615,7 +649,7 @@ fail:
     return -1;
 }
 
-static int dxva2_init_decoder(struct lavc_ctx *s, int fmt, int w, int h)
+static int dxva2_init_decoder(struct lavc_ctx *s, int w, int h)
 {
     DXVA2Context *ctx = s->hwdec_priv;
 
@@ -642,6 +676,7 @@ static int dxva2_init_decoder(struct lavc_ctx *s, int fmt, int w, int h)
 static int probe(struct vd_lavc_hwdec *hwdec, struct mp_hwdec_info *info,
                  const char *decoder)
 {
+    hwdec_request_api(info, "dxva2"); // we can do without too
     for (int i = 0; dxva2_modes[i].guid; i++) {
         const dxva2_mode *mode = &dxva2_modes[i];
         if (mp_codec_to_av_codec_id(decoder) == mode->codec)
