@@ -26,12 +26,12 @@
 #include "video/decode/lavc.h"
 #include "config.h"
 
-
 static int probe(struct vd_lavc_hwdec *hwdec, struct mp_hwdec_info *info,
                  const char *decoder)
 {
-    hwdec_request_api(info, "vda");
-
+    hwdec_request_api(info, "videotoolbox");
+    if (!info || !info->hwctx)
+        return HWDEC_ERR_NO_CTX;
     if (mp_codec_to_av_codec_id(decoder) != AV_CODEC_ID_H264)
         return HWDEC_ERR_NO_CODEC;
     return 0;
@@ -73,10 +73,17 @@ static void print_vda_error(struct mp_log *log, int lev, char *message,
     mp_msg(log, lev, "%s: %d\n", message, error_code);
 }
 
-static int init_decoder(struct lavc_ctx *ctx, int fmt, int w, int h)
+static int init_decoder(struct lavc_ctx *ctx, int w, int h)
 {
     av_vda_default_free(ctx->avctx);
+#if HAVE_VDA_DEFAULT_INIT2
+    AVVDAContext *vdactx = av_vda_alloc_context();
+    vdactx->cv_pix_fmt_type = kCVPixelFormatType_420YpCbCr8BiPlanarVideoRange;
+    int err = av_vda_default_init2(ctx->avctx, vdactx);
+#else
     int err = av_vda_default_init(ctx->avctx);
+#endif
+
     if (err < 0) {
         print_vda_error(ctx->log, MSGL_ERR, "failed to init VDA decoder", err);
         return -1;
@@ -90,6 +97,16 @@ static void uninit(struct lavc_ctx *ctx)
         av_vda_default_free(ctx->avctx);
 }
 
+static struct mp_image *process_image(struct lavc_ctx *ctx, struct mp_image *img)
+{
+    // Same representation. IMGFMT_VDA is only needed to select the libavcodec
+    // hwaccel driver.
+    if (img->imgfmt == IMGFMT_VDA)
+        mp_image_setfmt(img, IMGFMT_VIDEOTOOLBOX);
+
+    return img;
+}
+
 const struct vd_lavc_hwdec mp_vd_lavc_vda = {
     .type = HWDEC_VDA,
     .image_format = IMGFMT_VDA,
@@ -97,4 +114,5 @@ const struct vd_lavc_hwdec mp_vd_lavc_vda = {
     .init = init,
     .uninit = uninit,
     .init_decoder = init_decoder,
+    .process_image = process_image,
 };
