@@ -1,19 +1,18 @@
 /*
- * This file is part of MPlayer.
+ * This file is part of mpv.
  *
- * MPlayer is free software; you can redistribute it and/or modify
+ * mpv is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * MPlayer is distributed in the hope that it will be useful,
+ * mpv is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with MPlayer; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <libavutil/pixdesc.h>
@@ -88,37 +87,48 @@ static const struct {
 
     {IMGFMT_XYZ12,   AV_PIX_FMT_XYZ12},
 
-    // ffmpeg only
-#if LIBAVUTIL_VERSION_MICRO >= 100
+#ifdef AV_PIX_FMT_YUV420P12
     {IMGFMT_420P12,  AV_PIX_FMT_YUV420P12},
     {IMGFMT_420P14,  AV_PIX_FMT_YUV420P14},
     {IMGFMT_422P12,  AV_PIX_FMT_YUV422P12},
     {IMGFMT_422P14,  AV_PIX_FMT_YUV422P14},
     {IMGFMT_444P12,  AV_PIX_FMT_YUV444P12},
     {IMGFMT_444P14,  AV_PIX_FMT_YUV444P14},
+#endif
 
+#ifdef AV_PIX_FMT_RGBA64
     {IMGFMT_RGBA64,  AV_PIX_FMT_RGBA64},
     {IMGFMT_BGRA64,  AV_PIX_FMT_BGRA64},
+#endif
 
+#if LIBAVUTIL_VERSION_MICRO >= 100
     {IMGFMT_BGR0,  AV_PIX_FMT_BGR0},
     {IMGFMT_0RGB,  AV_PIX_FMT_0RGB},
     {IMGFMT_RGB0,  AV_PIX_FMT_RGB0},
     {IMGFMT_0BGR,  AV_PIX_FMT_0BGR},
-    {IMGFMT_BGR0,  AV_PIX_FMT_BGR0},
 #else
     {IMGFMT_BGR0,  AV_PIX_FMT_BGRA},
     {IMGFMT_0RGB,  AV_PIX_FMT_ARGB},
     {IMGFMT_RGB0,  AV_PIX_FMT_RGBA},
     {IMGFMT_0BGR,  AV_PIX_FMT_ABGR},
-    {IMGFMT_BGR0,  AV_PIX_FMT_BGRA},
+#endif
+
+#ifdef AV_PIX_FMT_YA16
+    {IMGFMT_YA16,  AV_PIX_FMT_YA16},
 #endif
 
     {IMGFMT_VDPAU, AV_PIX_FMT_VDPAU},
 #if HAVE_VDA_HWACCEL
     {IMGFMT_VDA,   AV_PIX_FMT_VDA},
+#endif
+#if HAVE_VIDEOTOOLBOX_HWACCEL
+    {IMGFMT_VIDEOTOOLBOX,   AV_PIX_FMT_VIDEOTOOLBOX},
 #endif
     {IMGFMT_VAAPI, AV_PIX_FMT_VAAPI_VLD},
     {IMGFMT_DXVA2, AV_PIX_FMT_DXVA2_VLD},
+#if HAVE_AV_PIX_FMT_MMAL
+    {IMGFMT_MMAL, AV_PIX_FMT_MMAL},
+#endif
 
     {0, AV_PIX_FMT_NONE}
 };
