@@ -1,18 +1,18 @@
 /*
- * This file is part of mplayer.
+ * This file is part of mpv.
  *
- * mplayer is free software; you can redistribute it and/or modify
+ * mpv is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * mplayer is distributed in the hope that it will be useful,
+ * mpv is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with mplayer.  If not, see <http://www.gnu.org/licenses/>.
+ * with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stdio.h>
@@ -42,14 +42,13 @@
 
 const struct image_writer_opts image_writer_opts_defaults = {
     .format = "jpg",
+    .high_bit_depth = 1,
     .png_compression = 7,
     .png_filter = 5,
     .jpeg_quality = 90,
-    .jpeg_optimize = 100,
     .jpeg_smooth = 0,
-    .jpeg_dpi = 72,
-    .jpeg_progressive = 0,
-    .jpeg_baseline = 1,
+    .jpeg_source_chroma = 1,
+    .tag_csp = 0,
 };
 
 #define OPT_BASE_STRUCT struct image_writer_opts
@@ -57,14 +56,13 @@ const struct image_writer_opts image_writer_opts_defaults = {
 const struct m_sub_options image_writer_conf = {
     .opts = (const m_option_t[]) {
         OPT_INTRANGE("jpeg-quality", jpeg_quality, 0, 0, 100),
-        OPT_INTRANGE("jpeg-optimize", jpeg_optimize, 0, 0, 100),
         OPT_INTRANGE("jpeg-smooth", jpeg_smooth, 0, 0, 100),
-        OPT_INTRANGE("jpeg-dpi", jpeg_dpi, M_OPT_MIN, 1, 99999),
-        OPT_FLAG("jpeg-progressive", jpeg_progressive, 0),
-        OPT_FLAG("jpeg-baseline", jpeg_baseline, 0),
+        OPT_FLAG("jpeg-source-chroma", jpeg_source_chroma, 0),
         OPT_INTRANGE("png-compression", png_compression, 0, 0, 9),
         OPT_INTRANGE("png-filter", png_filter, 0, 0, 5),
         OPT_STRING("format", format, 0),
+        OPT_FLAG("high-bit-depth", high_bit_depth, 0),
+        OPT_FLAG("tag-colorspace", tag_csp, 0),
         {0},
     },
     .size = sizeof(struct image_writer_opts),
@@ -75,18 +73,19 @@ struct image_writer_ctx {
     struct mp_log *log;
     const struct image_writer_opts *opts;
     const struct img_writer *writer;
+    struct mp_imgfmt_desc original_format;
 };
 
 struct img_writer {
     const char *file_ext;
-    int (*write)(struct image_writer_ctx *ctx, mp_image_t *image, FILE *fp);
+    bool (*write)(struct image_writer_ctx *ctx, mp_image_t *image, FILE *fp);
     const int *pixfmts;
     int lavc_codec;
 };
 
-static int write_lavc(struct image_writer_ctx *ctx, mp_image_t *image, FILE *fp)
+static bool write_lavc(struct image_writer_ctx *ctx, mp_image_t *image, FILE *fp)
 {
-    int success = 0;
+    bool success = 0;
     AVFrame *pic = NULL;
     AVPacket pkt = {0};
     int got_output = 0;
@@ -131,6 +130,10 @@ static int write_lavc(struct image_writer_ctx *ctx, mp_image_t *image, FILE *fp)
     pic->format = avctx->pix_fmt;
     pic->width = avctx->width;
     pic->height = avctx->height;
+    if (ctx->opts->tag_csp) {
+        pic->color_primaries = mp_csp_prim_to_avcol_pri(image->params.primaries);
+        pic->color_trc = mp_csp_trc_to_avcol_trc(image->params.gamma);
+    }
     int ret = avcodec_encode_video2(avctx, &pkt, pic, &got_output);
     if (ret < 0)
         goto error_exit;
@@ -158,7 +161,7 @@ static void write_jpeg_error_exit(j_common_ptr cinfo)
   longjmp(*(jmp_buf*)cinfo->client_data, 1);
 }
 
-static int write_jpeg(struct image_writer_ctx *ctx, mp_image_t *image, FILE *fp)
+static bool write_jpeg(struct image_writer_ctx *ctx, mp_image_t *image, FILE *fp)
 {
     struct jpeg_compress_struct cinfo;
     struct jpeg_error_mgr jerr;
@@ -170,7 +173,7 @@ static int write_jpeg(struct image_writer_ctx *ctx, mp_image_t *image, FILE *fp)
     cinfo.client_data = &error_return_jmpbuf;
     if (setjmp(cinfo.client_data)) {
         jpeg_destroy_compress(&cinfo);
-        return 0;
+        return false;
     }
 
     jpeg_create_compress(&cinfo);
@@ -184,18 +187,15 @@ static int write_jpeg(struct image_writer_ctx *ctx, mp_image_t *image, FILE *fp)
     cinfo.write_JFIF_header = TRUE;
     cinfo.JFIF_major_version = 1;
     cinfo.JFIF_minor_version = 2;
-    cinfo.density_unit = 1; /* 0=unknown, 1=dpi, 2=dpcm */
-    cinfo.X_density = ctx->opts->jpeg_dpi;
-    cinfo.Y_density = ctx->opts->jpeg_dpi;
-    cinfo.write_Adobe_marker = TRUE;
 
     jpeg_set_defaults(&cinfo);
-    jpeg_set_quality(&cinfo, ctx->opts->jpeg_quality, ctx->opts->jpeg_baseline);
-    cinfo.optimize_coding = ctx->opts->jpeg_optimize;
+    jpeg_set_quality(&cinfo, ctx->opts->jpeg_quality, 0);
     cinfo.smoothing_factor = ctx->opts->jpeg_smooth;
 
-    if (ctx->opts->jpeg_progressive)
-        jpeg_simple_progression(&cinfo);
+    if (ctx->opts->jpeg_source_chroma) {
+        cinfo.comp_info[0].h_samp_factor = 1 << ctx->original_format.chroma_xs;
+        cinfo.comp_info[0].v_samp_factor = 1 << ctx->original_format.chroma_ys;
+    }
 
     jpeg_start_compress(&cinfo, TRUE);
 
@@ -210,27 +210,55 @@ static int write_jpeg(struct image_writer_ctx *ctx, mp_image_t *image, FILE *fp)
 
     jpeg_destroy_compress(&cinfo);
 
-    return 1;
+    return true;
 }
 
 #endif
 
+static int get_encoder_format(struct AVCodec *codec, int srcfmt, bool highdepth)
+{
+    const enum AVPixelFormat *pix_fmts = codec->pix_fmts;
+    int current = 0;
+    for (int n = 0; pix_fmts && pix_fmts[n] != AV_PIX_FMT_NONE; n++) {
+        int fmt = pixfmt2imgfmt(pix_fmts[n]);
+        if (!fmt)
+            continue;
+        // Ignore formats larger than 8 bit per pixel.
+        if (!highdepth && IMGFMT_RGB_DEPTH(fmt) > 32)
+            continue;
+        current = current ? mp_imgfmt_select_best(current, fmt, srcfmt) : fmt;
+    }
+    return current;
+}
+
+static int get_target_format(struct image_writer_ctx *ctx, int srcfmt)
+{
+    if (!ctx->writer->lavc_codec)
+        goto unknown;
+
+    struct AVCodec *codec = avcodec_find_encoder(ctx->writer->lavc_codec);
+    if (!codec)
+        goto unknown;
+
+    int target = get_encoder_format(codec, srcfmt, ctx->opts->high_bit_depth);
+    if (!target)
+        target = get_encoder_format(codec, srcfmt, true);
+
+    if (!target)
+        goto unknown;
+
+    return target;
+
+unknown:
+    return IMGFMT_RGB24;
+}
+
 static const struct img_writer img_writers[] = {
     { "png", write_lavc, .lavc_codec = AV_CODEC_ID_PNG },
     { "ppm", write_lavc, .lavc_codec = AV_CODEC_ID_PPM },
-    { "pgm", write_lavc,
-      .lavc_codec = AV_CODEC_ID_PGM,
-      .pixfmts = (const int[]) { IMGFMT_Y8, 0 },
-    },
-    { "pgmyuv", write_lavc,
-      .lavc_codec = AV_CODEC_ID_PGMYUV,
-      .pixfmts = (const int[]) { IMGFMT_420P, 0 },
-    },
-    { "tga", write_lavc,
-      .lavc_codec = AV_CODEC_ID_TARGA,
-      .pixfmts = (const int[]) { IMGFMT_BGR24, IMGFMT_BGRA, IMGFMT_RGB555,
-                                 IMGFMT_Y8, 0},
-    },
+    { "pgm", write_lavc, .lavc_codec = AV_CODEC_ID_PGM },
+    { "pgmyuv", write_lavc, .lavc_codec = AV_CODEC_ID_PGMYUV },
+    { "tga", write_lavc, .lavc_codec = AV_CODEC_ID_TARGA },
 #if HAVE_JPEG
     { "jpg", write_jpeg },
     { "jpeg", write_jpeg },
@@ -260,65 +288,61 @@ const char *image_writer_file_ext(const struct image_writer_opts *opts)
     return get_writer(opts)->file_ext;
 }
 
-int write_image(struct mp_image *image, const struct image_writer_opts *opts,
-                const char *filename, struct mp_log *log)
+struct mp_image *convert_image(struct mp_image *image, int destfmt,
+                               struct mp_log *log)
 {
-    struct mp_image *allocated_image = NULL;
-    struct image_writer_opts defs = image_writer_opts_defaults;
     int d_w = image->params.d_w;
     int d_h = image->params.d_h;
     bool is_anamorphic = image->w != d_w || image->h != d_h;
 
+    // Caveat: no colorspace/levels conversion done if pixel formats equal
+    //         it's unclear what colorspace/levels the target wants
+    if (image->imgfmt == destfmt && !is_anamorphic)
+        return mp_image_new_ref(image);
+
+    struct mp_image *dst = mp_image_alloc(destfmt, d_w, d_h);
+    if (!dst) {
+        mp_err(log, "Out of memory.\n");
+        return NULL;
+    }
+    mp_image_copy_attributes(dst, image);
+
+    if (mp_image_swscale(dst, image, mp_sws_hq_flags) < 0) {
+        mp_err(log, "Error when converting image.\n");
+        talloc_free(dst);
+        return NULL;
+    }
+
+    return dst;
+}
+
+bool write_image(struct mp_image *image, const struct image_writer_opts *opts,
+                const char *filename, struct mp_log *log)
+{
+    struct image_writer_opts defs = image_writer_opts_defaults;
     if (!opts)
         opts = &defs;
 
     const struct img_writer *writer = get_writer(opts);
-    struct image_writer_ctx ctx = { log, opts, writer };
-    int destfmt = IMGFMT_RGB24;
+    struct image_writer_ctx ctx = { log, opts, writer, image->fmt };
+    int destfmt = get_target_format(&ctx, image->imgfmt);
 
-    if (writer->pixfmts) {
-        destfmt = writer->pixfmts[0];   // default to first pixel format
-        for (const int *fmt = writer->pixfmts; *fmt; fmt++) {
-            if (*fmt == image->imgfmt) {
-                destfmt = *fmt;
-                break;
-            }
-        }
-    }
-
-    // Caveat: no colorspace/levels conversion done if pixel formats equal
-    //         it's unclear what colorspace/levels the target wants
-    if (image->imgfmt != destfmt || is_anamorphic) {
-        struct mp_image *dst = mp_image_alloc(destfmt, d_w, d_h);
-        if (!dst) {
-            mp_err(log, "Out of memory.\n");
-            return 0;
-        }
-        mp_image_copy_attributes(dst, image);
-
-        if (mp_image_swscale(dst, image, mp_sws_hq_flags) < 0) {
-            mp_err(log, "Error when converting image.\n");
-            talloc_free(dst);
-            return 0;
-        }
-
-        allocated_image = dst;
-        image = dst;
-    }
+    struct mp_image *dst = convert_image(image, destfmt, log);
+    if (!dst)
+        return false;
 
     FILE *fp = fopen(filename, "wb");
-    int success = 0;
+    bool success = false;
     if (fp == NULL) {
         mp_err(log, "Error opening '%s' for writing!\n", filename);
     } else {
-        success = writer->write(&ctx, image, fp);
+        success = writer->write(&ctx, dst, fp);
         success = !fclose(fp) && success;
         if (!success)
             mp_err(log, "Error writing file '%s'!\n", filename);
     }
 
-    talloc_free(allocated_image);
-
+    talloc_free(dst);
     return success;
 }
 
