@@ -1,22 +1,22 @@
 /*
  * video output driver for SDL 2.0+
+ *
  * Copyright (C) 2012 Rudolf Polzer <divVerent@xonotic.org>
  *
  * This file is part of mpv.
  *
- * MPlayer is free software; you can redistribute it and/or modify
+ * mpv is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * MPlayer is distributed in the hope that it will be useful,
+ * mpv is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with MPlayer; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stdio.h>
@@ -41,7 +41,6 @@
 #include "sub/osd.h"
 
 #include "video/mp_image.h"
-#include "video/vfcap.h"
 
 #include "win_state.h"
 #include "config.h"
@@ -173,14 +172,12 @@ struct priv {
     SDL_Texture *tex;
     int tex_swapped;
     struct mp_image_params params;
-    mp_image_t *ssmpi;
     struct mp_rect src_rect;
     struct mp_rect dst_rect;
     struct mp_osd_res osd_res;
     struct formatmap_entry osd_format;
     struct osd_bitmap_surface {
-        int bitmap_id;
-        int bitmap_pos_id;
+        int change_id;
         struct osd_target {
             SDL_Rect source;
             SDL_Rect dest;
@@ -577,7 +574,7 @@ static int wait_events(struct vo *vo, int64_t until_time_us)
             if (sdl_mod & (KMOD_LCTRL | KMOD_RCTRL))
                 mpv_mod |= MP_KEY_MODIFIER_CTRL;
             if ((sdl_mod & KMOD_LALT) ||
-                (sdl_mod & KMOD_RALT) && !mp_input_use_alt_gr(vo->input_ctx))
+                ((sdl_mod & KMOD_RALT) && !mp_input_use_alt_gr(vo->input_ctx)))
                 mpv_mod |= MP_KEY_MODIFIER_ALT;
             if (sdl_mod & (KMOD_LGUI | KMOD_RGUI))
                 mpv_mod |= MP_KEY_MODIFIER_META;
@@ -639,7 +636,6 @@ static void uninit(struct vo *vo)
 {
     struct priv *vc = vo->priv;
     destroy_renderer(vo);
-    talloc_free(vc->ssmpi);
     SDL_QuitSubSystem(SDL_INIT_VIDEO);
     talloc_free(vc);
 }
@@ -705,7 +701,7 @@ static void generate_osd_part(struct vo *vo, struct sub_bitmaps *imgs)
     if (imgs->format == SUBBITMAP_EMPTY || imgs->num_parts == 0)
         return;
 
-    if (imgs->bitmap_pos_id == sfc->bitmap_pos_id)
+    if (imgs->change_id == sfc->change_id)
         return;
 
     if (imgs->num_parts > sfc->targets_size) {
@@ -728,49 +724,46 @@ static void generate_osd_part(struct vo *vo, struct sub_bitmaps *imgs)
             bmp->x, bmp->y, bmp->dw, bmp->dh
         };
 
-        if (imgs->bitmap_id != sfc->bitmap_id || !target->tex) {
-            // tex: alpha blended texture
-            if (target->tex) {
-                SDL_DestroyTexture(target->tex);
-                target->tex = NULL;
-            }
-            if (!target->tex)
-                target->tex = SDL_CreateTexture(vc->renderer,
-                        vc->osd_format.sdl, SDL_TEXTUREACCESS_STREAMING,
-                        bmp->w, bmp->h);
-            if (!target->tex) {
-                MP_ERR(vo, "Could not create texture\n");
-            }
-            if (target->tex) {
-                SDL_SetTextureBlendMode(target->tex,
-                                        SDL_BLENDMODE_BLEND);
-                SDL_SetTextureColorMod(target->tex, 0, 0, 0);
-                subbitmap_to_texture(vo, target->tex, bmp, 0); // RGBA -> 000A
-            }
+        // tex: alpha blended texture
+        if (target->tex) {
+            SDL_DestroyTexture(target->tex);
+            target->tex = NULL;
+        }
+        if (!target->tex)
+            target->tex = SDL_CreateTexture(vc->renderer,
+                    vc->osd_format.sdl, SDL_TEXTUREACCESS_STREAMING,
+                    bmp->w, bmp->h);
+        if (!target->tex) {
+            MP_ERR(vo, "Could not create texture\n");
+        }
+        if (target->tex) {
+            SDL_SetTextureBlendMode(target->tex,
+                                    SDL_BLENDMODE_BLEND);
+            SDL_SetTextureColorMod(target->tex, 0, 0, 0);
+            subbitmap_to_texture(vo, target->tex, bmp, 0); // RGBA -> 000A
+        }
 
-            // tex2: added texture
-            if (target->tex2) {
-                SDL_DestroyTexture(target->tex2);
-                target->tex2 = NULL;
-            }
-            if (!target->tex2)
-                target->tex2 = SDL_CreateTexture(vc->renderer,
-                        vc->osd_format.sdl, SDL_TEXTUREACCESS_STREAMING,
-                        bmp->w, bmp->h);
-            if (!target->tex2) {
-                MP_ERR(vo, "Could not create texture\n");
-            }
-            if (target->tex2) {
-                SDL_SetTextureBlendMode(target->tex2,
-                                        SDL_BLENDMODE_ADD);
-                subbitmap_to_texture(vo, target->tex2, bmp,
-                                     0xFF000000); // RGBA -> RGB1
-            }
+        // tex2: added texture
+        if (target->tex2) {
+            SDL_DestroyTexture(target->tex2);
+            target->tex2 = NULL;
+        }
+        if (!target->tex2)
+            target->tex2 = SDL_CreateTexture(vc->renderer,
+                    vc->osd_format.sdl, SDL_TEXTUREACCESS_STREAMING,
+                    bmp->w, bmp->h);
+        if (!target->tex2) {
+            MP_ERR(vo, "Could not create texture\n");
+        }
+        if (target->tex2) {
+            SDL_SetTextureBlendMode(target->tex2,
+                                    SDL_BLENDMODE_ADD);
+            subbitmap_to_texture(vo, target->tex2, bmp,
+                                    0xFF000000); // RGBA -> RGB1
         }
     }
 
-    sfc->bitmap_id = imgs->bitmap_id;
-    sfc->bitmap_pos_id = imgs->bitmap_pos_id;
+    sfc->change_id = imgs->change_id;
 }
 
 static void draw_osd_part(struct vo *vo, int index)
@@ -847,16 +840,15 @@ static int preinit(struct vo *vo)
     return 0;
 }
 
-static int query_format(struct vo *vo, uint32_t format)
+static int query_format(struct vo *vo, int format)
 {
     struct priv *vc = vo->priv;
     int i, j;
-    int cap = VFCAP_CSP_SUPPORTED;
     for (i = 0; i < vc->renderer_info.num_texture_formats; ++i)
         for (j = 0; j < sizeof(formats) / sizeof(formats[0]); ++j)
             if (vc->renderer_info.texture_formats[i] == formats[j].sdl)
                 if (format == formats[j].mpv)
-                    return cap;
+                    return 1;
     return 0;
 }
 
@@ -916,8 +908,7 @@ static void draw_image(struct vo *vo, mp_image_t *mpi)
 
         SDL_UnlockTexture(vc->tex);
 
-        talloc_free(vc->ssmpi);
-        vc->ssmpi = mpi;
+        talloc_free(mpi);
     }
 
     SDL_Rect src, dst;
@@ -943,12 +934,6 @@ static void draw_image(struct vo *vo, mp_image_t *mpi)
     draw_osd(vo);
 }
 
-static struct mp_image *get_screenshot(struct vo *vo)
-{
-    struct priv *vc = vo->priv;
-    return vc->ssmpi ? mp_image_new_ref(vc->ssmpi) : NULL;
-}
-
 static struct mp_image *get_window_screenshot(struct vo *vo)
 {
     struct priv *vc = vo->priv;
@@ -1001,6 +986,7 @@ static int control(struct vo *vo, uint32_t request, void *data)
 
     switch (request) {
     case VOCTRL_FULLSCREEN:
+        vo->opts->fullscreen = !vo->opts->fullscreen;
         set_fullscreen(vo);
         return 1;
     case VOCTRL_REDRAW_FRAME:
@@ -1019,14 +1005,9 @@ static int control(struct vo *vo, uint32_t request, void *data)
         struct voctrl_get_equalizer_args *args = data;
         return get_eq(vo, args->name, args->valueptr);
     }
-    case VOCTRL_SCREENSHOT: {
-        struct voctrl_screenshot_args *args = data;
-        if (args->full_window)
-            args->out_image = get_window_screenshot(vo);
-        else
-            args->out_image = get_screenshot(vo);
+    case VOCTRL_SCREENSHOT_WIN:
+        *(struct mp_image **)data = get_window_screenshot(vo);
         return true;
-    }
     case VOCTRL_SET_CURSOR_VISIBILITY:
         SDL_ShowCursor(*(bool *)data);
         return true;
