@@ -63,6 +63,8 @@
 #define CK_SRC_SET           1 // use and set specified / default colorkey
 #define CK_SRC_CUR           2 // use current colorkey (get it from xv)
 
+#define MAX_BUFFERS 10
+
 struct xvctx {
     struct xv_ck_info_s {
         int method; // CK_METHOD_* constants
@@ -72,13 +74,14 @@ struct xvctx {
     unsigned long xv_colorkey;
     int xv_port;
     int cfg_xv_adaptor;
+    int cfg_buffers;
     XvAdaptorInfo *ai;
     XvImageFormatValues *fo;
     unsigned int formats, adaptors, xv_format;
     int current_buf;
     int current_ip_buf;
     int num_buffers;
-    XvImage *xvimage[2];
+    XvImage *xvimage[MAX_BUFFERS];
     struct mp_image *original_image;
     uint32_t image_width;
     uint32_t image_height;
@@ -87,9 +90,11 @@ struct xvctx {
     struct mp_rect src_rect;
     struct mp_rect dst_rect;
     uint32_t max_width, max_height; // zero means: not set
+    GC f_gc;    // used to paint background
+    GC vo_gc;   // used to paint video
     int Shmem_Flag;
 #if HAVE_SHM && HAVE_XEXT
-    XShmSegmentInfo Shminfo[2];
+    XShmSegmentInfo Shminfo[MAX_BUFFERS];
     int Shm_Warned_Slow;
 #endif
 };
@@ -378,11 +383,11 @@ static void xv_draw_colorkey(struct vo *vo, const struct mp_rect *rc)
     if (ctx->xv_ck_info.method == CK_METHOD_MANUALFILL ||
         ctx->xv_ck_info.method == CK_METHOD_BACKGROUND)
     {
-        if (!x11->vo_gc)
+        if (!ctx->vo_gc)
             return;
         //less tearing than XClearWindow()
-        XSetForeground(x11->display, x11->vo_gc, ctx->xv_colorkey);
-        XFillRectangle(x11->display, x11->window, x11->vo_gc, rc->x0, rc->y0,
+        XSetForeground(x11->display, ctx->vo_gc, ctx->xv_colorkey);
+        XFillRectangle(x11->display, x11->window, ctx->vo_gc, rc->x0, rc->y0,
                        rc->x1 - rc->x0, rc->y1 - rc->y0);
     }
 }
@@ -396,6 +401,38 @@ static void read_xv_csp(struct vo *vo)
         ctx->cached_csp = bt709_enabled == 100 ? MP_CSP_BT_709 : MP_CSP_BT_601;
 }
 
+
+static void fill_rect(struct vo *vo, GC gc, int x0, int y0, int x1, int y1)
+{
+    struct vo_x11_state *x11 = vo->x11;
+
+    x0 = MPMAX(x0, 0);
+    y0 = MPMAX(y0, 0);
+    x1 = MPMIN(x1, vo->dwidth);
+    y1 = MPMIN(y1, vo->dheight);
+
+    if (x11->window && x1 > x0 && y1 > y0)
+        XFillRectangle(x11->display, x11->window, gc, x0, y0, x1 - x0, y1 - y0);
+}
+
+// Clear everything outside of rc with the background color
+static void vo_x11_clear_background(struct vo *vo, const struct mp_rect *rc)
+{
+    struct vo_x11_state *x11 = vo->x11;
+    struct xvctx *ctx = vo->priv;
+    GC gc = ctx->f_gc;
+
+    int w = vo->dwidth;
+    int h = vo->dheight;
+
+    fill_rect(vo, gc, 0,      0,      w,      rc->y0); // top
+    fill_rect(vo, gc, 0,      rc->y1, w,      h);      // bottom
+    fill_rect(vo, gc, 0,      rc->y0, rc->x0, rc->y1); // left
+    fill_rect(vo, gc, rc->x1, rc->y0, w,      rc->y1); // right
+
+    XFlush(x11->display);
+}
+
 static void resize(struct vo *vo)
 {
     struct xvctx *ctx = vo->priv;
@@ -454,6 +491,12 @@ static int reconfig(struct vo *vo, struct mp_image_params *params, int flags)
 
     vo_x11_config_vo_window(vo, NULL, flags, "xv");
 
+    if (!ctx->f_gc && !ctx->vo_gc) {
+        ctx->f_gc = XCreateGC(x11->display, x11->window, 0, 0);
+        ctx->vo_gc = XCreateGC(x11->display, x11->window, 0, NULL);
+        XSetForeground(x11->display, ctx->f_gc, 0);
+    }
+
     if (ctx->xv_ck_info.method == CK_METHOD_BACKGROUND)
         XSetWindowBackground(x11->display, x11->window, ctx->xv_colorkey);
 
@@ -463,7 +506,7 @@ static int reconfig(struct vo *vo, struct mp_image_params *params, int flags)
     for (i = 0; i < ctx->num_buffers; i++)
         deallocate_xvimage(vo, i);
 
-    ctx->num_buffers = 2;
+    ctx->num_buffers = ctx->cfg_buffers;
 
     for (i = 0; i < ctx->num_buffers; i++) {
         if (!allocate_xvimage(vo, i)) {
@@ -574,7 +617,7 @@ static inline void put_xvimage(struct vo *vo, XvImage *xvi)
     int sw = src->x1 - src->x0, sh = src->y1 - src->y0;
 #if HAVE_SHM && HAVE_XEXT
     if (ctx->Shmem_Flag) {
-        XvShmPutImage(x11->display, ctx->xv_port, x11->window, x11->vo_gc, xvi,
+        XvShmPutImage(x11->display, ctx->xv_port, x11->window, ctx->vo_gc, xvi,
                       src->x0, src->y0, sw, sh,
                       dst->x0, dst->y0, dw, dh,
                       True);
@@ -582,7 +625,7 @@ static inline void put_xvimage(struct vo *vo, XvImage *xvi)
     } else
 #endif
     {
-        XvPutImage(x11->display, ctx->xv_port, x11->window, x11->vo_gc, xvi,
+        XvPutImage(x11->display, ctx->xv_port, x11->window, ctx->vo_gc, xvi,
                    src->x0, src->y0, sw, sh,
                    dst->x0, dst->y0, dw, dh);
     }
@@ -699,6 +742,10 @@ static void uninit(struct vo *vo)
     }
     for (i = 0; i < ctx->num_buffers; i++)
         deallocate_xvimage(vo, i);
+    if (ctx->f_gc != None)
+        XFreeGC(vo->x11->display, ctx->f_gc);
+    if (ctx->vo_gc != None)
+        XFreeGC(vo->x11->display, ctx->vo_gc);
     // uninit() shouldn't get called unless initialization went past vo_init()
     vo_x11_uninit(vo);
 }
@@ -848,6 +895,7 @@ const struct vo_driver video_out_xv = {
         .xv_ck_info = {CK_METHOD_MANUALFILL, CK_SRC_CUR},
         .colorkey = 0x0000ff00, // default colorkey is green
                     // (0xff000000 means that colorkey has been disabled)
+        .cfg_buffers = 2,
     },
     .options = (const struct m_option[]) {
         OPT_INT("port", xv_port, M_OPT_MIN, .min = 0),
@@ -862,6 +910,7 @@ const struct vo_driver video_out_xv = {
                     {"auto", CK_METHOD_AUTOPAINT})),
         OPT_INT("colorkey", colorkey, 0),
         OPT_FLAG_STORE("no-colorkey", colorkey, 0, 0x1000000),
+        OPT_INTRANGE("buffers", cfg_buffers, 0, 1, MAX_BUFFERS),
         {0}
     },
 };
