@@ -100,38 +100,23 @@ def build(ctx):
         ( "audio/decode/ad_spdif.c" ),
         ( "audio/decode/dec_audio.c" ),
         ( "audio/filter/af.c" ),
-        ( "audio/filter/af_bs2b.c",              "libbs2b" ),
-        ( "audio/filter/af_center.c" ),
         ( "audio/filter/af_channels.c" ),
-        ( "audio/filter/af_convert24.c" ),
-        ( "audio/filter/af_convertsignendian.c" ),
         ( "audio/filter/af_delay.c" ),
         ( "audio/filter/af_drc.c" ),
-        ( "audio/filter/af_dummy.c" ),
         ( "audio/filter/af_equalizer.c" ),
-        ( "audio/filter/af_export.c" ),
-        ( "audio/filter/af_extrastereo.c" ),
         ( "audio/filter/af_format.c" ),
-        ( "audio/filter/af_hrtf.c" ),
-        ( "audio/filter/af_karaoke.c" ),
-        ( "audio/filter/af_ladspa.c",            "ladspa" ),
         ( "audio/filter/af_lavcac3enc.c" ),
         ( "audio/filter/af_lavfi.c",             "libavfilter" ),
         ( "audio/filter/af_lavrresample.c" ),
         ( "audio/filter/af_pan.c" ),
         ( "audio/filter/af_rubberband.c",        "rubberband" ),
         ( "audio/filter/af_scaletempo.c" ),
-        ( "audio/filter/af_sinesuppress.c" ),
-        ( "audio/filter/af_sub.c" ),
-        ( "audio/filter/af_surround.c" ),
-        ( "audio/filter/af_sweep.c" ),
         ( "audio/filter/af_volume.c" ),
-        ( "audio/filter/filter.c" ),
         ( "audio/filter/tools.c" ),
-        ( "audio/filter/window.c" ),
         ( "audio/out/ao.c" ),
         ( "audio/out/ao_alsa.c",                 "alsa" ),
         ( "audio/out/ao_coreaudio.c",            "coreaudio" ),
+        ( "audio/out/ao_coreaudio_chmap.c",      "coreaudio" ),
         ( "audio/out/ao_coreaudio_exclusive.c",  "coreaudio" ),
         ( "audio/out/ao_coreaudio_properties.c", "coreaudio" ),
         ( "audio/out/ao_coreaudio_utils.c",      "coreaudio" ),
@@ -166,11 +151,13 @@ def build(ctx):
 
         ## Demuxers
         ( "demux/codec_tags.c" ),
+        ( "demux/cue.c" ),
         ( "demux/demux.c" ),
         ( "demux/demux_cue.c" ),
         ( "demux/demux_disc.c" ),
         ( "demux/demux_edl.c" ),
         ( "demux/demux_lavf.c" ),
+        ( "demux/demux_libarchive.c",            "libarchive" ),
         ( "demux/demux_libass.c",                "libass"),
         ( "demux/demux_mf.c" ),
         ( "demux/demux_mkv.c" ),
@@ -215,7 +202,7 @@ def build(ctx):
         ( "player/client.c" ),
         ( "player/command.c" ),
         ( "player/configfiles.c" ),
-        ( "player/discnav.c" ),
+        ( "player/external_files.c" ),
         ( "player/loadfile.c" ),
         ( "player/main.c" ),
         ( "player/misc.c" ),
@@ -249,6 +236,7 @@ def build(ctx):
         ( "stream/stream_edl.c" ),
         ( "stream/stream_file.c" ),
         ( "stream/stream_lavf.c" ),
+        ( "stream/stream_libarchive.c",          "libarchive" ),
         ( "stream/stream_memory.c" ),
         ( "stream/stream_mf.c" ),
         ( "stream/stream_null.c" ),
@@ -264,7 +252,6 @@ def build(ctx):
         ( "sub/ass_mp.c",                        "libass"),
         ( "sub/dec_sub.c" ),
         ( "sub/draw_bmp.c" ),
-        ( "sub/find_subfiles.c" ),
         ( "sub/img_convert.c" ),
         ( "sub/osd.c" ),
         ( "sub/osd_dummy.c",                     "dummy-osd" ),
@@ -294,11 +281,11 @@ def build(ctx):
         ( "video/decode/vaapi.c",                "vaapi-hwaccel" ),
         ( "video/decode/vd_lavc.c" ),
         ( "video/decode/vda.c",                  "vda-hwaccel" ),
+        ( "video/decode/videotoolbox.c",         "videotoolbox-hwaccel" ),
         ( "video/decode/vdpau.c",                "vdpau-hwaccel" ),
         ( "video/filter/vf.c" ),
         ( "video/filter/vf_buffer.c" ),
         ( "video/filter/vf_crop.c" ),
-        ( "video/filter/vf_delogo.c",            "libavfilter"),
         ( "video/filter/vf_dlopen.c",            "dlopen" ),
         ( "video/filter/vf_dsize.c" ),
         ( "video/filter/vf_eq.c" ),
@@ -306,20 +293,18 @@ def build(ctx):
         ( "video/filter/vf_flip.c" ),
         ( "video/filter/vf_format.c" ),
         ( "video/filter/vf_gradfun.c",           "libavfilter"),
-        ( "video/filter/vf_hqdn3d.c",            "libavfilter"),
         ( "video/filter/vf_lavfi.c",             "libavfilter"),
         ( "video/filter/vf_mirror.c",            "libavfilter"),
         ( "video/filter/vf_noformat.c" ),
-        ( "video/filter/vf_noise.c",             "libavfilter"),
         ( "video/filter/vf_pullup.c",            "libavfilter"),
         ( "video/filter/vf_rotate.c",            "libavfilter"),
         ( "video/filter/vf_scale.c" ),
-        ( "video/filter/vf_stereo3d.c" ),
+        ( "video/filter/vf_stereo3d.c",          "libavfilter"),
         ( "video/filter/vf_sub.c" ),
-        ( "video/filter/vf_unsharp.c",           "libavfilter"),
         ( "video/filter/vf_vapoursynth.c",       "vapoursynth-core" ),
         ( "video/filter/vf_vavpp.c",             "vaapi-vpp"),
         ( "video/filter/vf_vdpaupp.c",           "vdpau" ),
+        ( "video/filter/vf_vdpaurb.c",           "vdpau" ),
         ( "video/filter/vf_yadif.c",             "libavfilter"),
         ( "video/out/aspect.c" ),
         ( "video/out/bitmap_packer.c" ),
@@ -329,21 +314,23 @@ def build(ctx):
         ( "video/out/cocoa_common.m",            "cocoa" ),
         ( "video/out/dither.c" ),
         ( "video/out/filter_kernels.c" ),
-        ( "video/out/gl_cocoa.c",                "gl-cocoa" ),
-        ( "video/out/gl_common.c",               "gl" ),
-        ( "video/out/gl_rpi.c",                  "rpi-gles" ),
-        ( "video/out/gl_hwdec.c",                "gl" ),
-        ( "video/out/gl_hwdec_vaglx.c",          "vaapi-glx" ),
-        ( "video/out/gl_hwdec_vda.c",            "vda-gl" ),
-        ( "video/out/gl_hwdec_vdpau.c",          "vdpau-gl-x11" ),
-        ( "video/out/gl_lcms.c",                 "gl" ),
-        ( "video/out/gl_osd.c",                  "gl" ),
-        ( "video/out/gl_utils.c",                "gl" ),
-        ( "video/out/gl_video.c",                "gl" ),
-        ( "video/out/gl_w32.c",                  "gl-win32" ),
-        ( "video/out/gl_wayland.c",              "gl-wayland" ),
-        ( "video/out/gl_x11.c",                  "gl-x11" ),
-        ( "video/out/gl_x11egl.c",               "egl-x11" ),
+        ( "video/out/opengl/cocoa.c",            "gl-cocoa" ),
+        ( "video/out/opengl/common.c",           "gl" ),
+        ( "video/out/opengl/rpi.c",              "rpi" ),
+        ( "video/out/opengl/hwdec.c",            "gl" ),
+        ( "video/out/opengl/hwdec_dxva2.c",      "gl-win32" ),
+        ( "video/out/opengl/hwdec_vaglx.c",      "vaapi-glx" ),
+        ( "video/out/opengl/hwdec_vda.c",        "videotoolbox-vda-gl" ),
+        ( "video/out/opengl/hwdec_vdpau.c",      "vdpau-gl-x11" ),
+        ( "video/out/opengl/lcms.c",             "gl" ),
+        ( "video/out/opengl/osd.c",              "gl" ),
+        ( "video/out/opengl/utils.c",            "gl" ),
+        ( "video/out/opengl/video.c",            "gl" ),
+        ( "video/out/opengl/video_shaders.c",    "gl" ),
+        ( "video/out/opengl/w32.c",              "gl-win32" ),
+        ( "video/out/opengl/wayland.c",          "gl-wayland" ),
+        ( "video/out/opengl/x11.c",              "gl-x11" ),
+        ( "video/out/opengl/x11egl.c",           "egl-x11" ),
         ( "video/out/vo.c" ),
         ( "video/out/vo_caca.c",                 "caca" ),
         ( "video/out/vo_drm.c",                  "drm" ),
@@ -358,7 +345,6 @@ def build(ctx):
         ( "video/out/vo_vaapi.c",                "vaapi" ),
         ( "video/out/vo_vdpau.c",                "vdpau" ),
         ( "video/out/vo_wayland.c",              "wayland" ),
-        ( "video/out/vo_x11.c" ,                 "x11" ),
         ( "video/out/vo_xv.c",                   "xv" ),
         ( "video/out/w32_common.c",              "win32" ),
         ( "video/out/wayland_common.c",          "wayland" ),
@@ -416,6 +402,12 @@ def build(ctx):
                 ctx.path.find_node('osdep/mpv.rc'),
                 ctx.path.find_node(node))
 
+        version = ctx.bldnode.find_node('version.h')
+        if version:
+            ctx.add_manual_dependency(
+                ctx.path.find_node('osdep/mpv.rc'),
+                version)
+
     if ctx.dependency_satisfied('cplayer') or ctx.dependency_satisfied('test'):
         ctx(
             target       = "objects",
